Package: vivaglint
Type: Package
Title: Analysis Tools for 'Viva Glint' Survey Data
Version: 0.1.1
Authors@R: 
    c(
        person("Eric", "Knudsen", , "ericknudsen@microsoft.com", role = c("aut", "cre")),
        person("Microsoft Corporation", role = "cph")
    )
Description: Provides functions for importing, validating, and analyzing 'Viva Glint'
    survey data exports, with optional API-based import via the 'Microsoft Graph'
    API. Includes tools for data reshaping, question-level analysis, multi-cycle
    comparisons, organizational hierarchy analysis, factor analysis, and
    correlation analysis. Harman (1960, ISBN: 0226316513); Husser (2017)
    <doi:10.1002/9781118901731.iecrm0048>.
License: MIT + file LICENSE
URL: https://github.com/microsoft/vivaglint
BugReports: https://github.com/microsoft/vivaglint/issues
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: dplyr (>= 1.0.0), tidyr (>= 1.0.0), readr (>= 2.0.0), stringr
        (>= 1.4.0), lubridate (>= 1.7.0), purrr (>= 0.3.0), rlang (>=
        0.4.0), httr (>= 1.4.0)
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, psych (>= 2.0.0),
        ggplot2 (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-22 21:30:09 UTC; ericknudsen
Author: Eric Knudsen [aut, cre],
  Microsoft Corporation [cph]
Maintainer: Eric Knudsen <ericknudsen@microsoft.com>
Repository: CRAN
Date/Publication: 2026-04-24 20:20:02 UTC
Built: R 4.6.0; ; 2026-04-30 05:54:18 UTC; unix

# Generated by roxygen2: do not edit by hand

export(aggregate_by_manager)
export(analyze_attrition)
export(analyze_by_attributes)
export(compare_cycles)
export(extract_questions)
export(extract_survey_factors)
export(get_correlations)
export(get_response_dist)
export(glint_setup)
export(join_attributes)
export(pivot_long)
export(read_glint_survey)
export(read_glint_survey_api)
export(search_comments)
export(split_survey_data)
export(summarize_survey)
importFrom(dplyr,"%>%")
importFrom(stats,cor)
importFrom(stats,cor.test)
importFrom(stats,sd)
importFrom(stats,setNames)

# vivaglint <img src="man\figures\vivaglint-badge.png" align="right" height="138" alt="vivaglint" />

R package for analyzing Viva Glint survey data with comprehensive statistical tools and organizational insights.

## Overview

**'vivaglint'** simplifies the analysis of 'Viva Glint' survey exports by providing a complete toolkit for data import, validation, statistical analysis, and reporting. The package handles the complexities of the 'Viva Glint' data structure and offers intuitive functions for common analysis tasks.

## Installation

Install the package from CRAN:

```r
# Install from CRAN
install.packages("vivaglint")
```

Install the development version from GitHub:

```r
# Install from GitHub
install.packages("devtools")
devtools::install_github("microsoft/vivaglint")
```

## Core Capabilities

### 1. **Data Import & Validation**
- Automatic validation of Glint export structure (CSV or API)
- Survey metadata extraction


### 2. **Survey Analysis**
- **Summary statistics**: Mean, SD, response rates, favorability percentages
- **Response distributions**: Detailed breakdown by response value
- **Cycle comparisons**: Track changes over time with delta calculations
- **Manager aggregations**: Roll up results by organizational hierarchy

### 3. **Advanced Analytics**
- **Correlation analysis**: Spearman, Pearson, and Kendall methods with significance testing
- **Factor analysis**: Identify latent constructs with multiple rotation options
- **Attrition prediction**: Link survey responses to employee turnover

## Quick Start

```r
library(vivaglint)

# 1. Load survey data
survey <- read_glint_survey("survey_export.csv")

# Or pull directly from the Viva Glint API
glint_setup(
  tenant_id = "your-tenant-id",
  client_id = "your-client-id",
  client_secret = "your-client-secret",
  experience_name = "your-experience-name"
)
survey <- read_glint_survey_api(
  survey_uuid = "your-survey-uuid",
  cycle_id = "your-cycle-id",
  emp_id_col = "EMP ID"
)

# 2. Get summary statistics (5-point scale)
summary <- summarize_survey(survey, scale_points = 5)
print(summary)

# 3. Analyze response distributions
distributions <- get_response_dist(survey)

# 4. Calculate question correlations
correlations <- get_correlations(survey, method = "spearman")

# 5. Aggregate by manager
manager_results <- aggregate_by_manager(survey, scale_points = 5)

# 6. Compare across cycles
comparison <- compare_cycles(survey1, survey2, survey3,
                            scale_points = 5,
                            cycle_names = c("Q1", "Q2", "Q3"))

# 7. Analyze employee attrition
attrition <- analyze_attrition(
  survey,
  attrition_file = "employee_attributes.csv",
  emp_id_col = "EMP ID",
  term_date_col = "Termination Date",
  scale_points = 5,
  time_periods = c(30, 90, 180)
)

# 8. Analyze by attributes
demo_results <- analyze_by_attributes(
  survey,
  attribute_file = "employee_attributes.csv",
  scale_points = 5,
  attribute_cols = c("Department", "Gender", "Tenure Group"),
  min_group_size = 10
)
```

## Key Functions

| Function | Purpose |
|----------|---------|
| `read_glint_survey()` | Import and validate Glint CSV exports |
| `read_glint_survey_api()` | Export and import survey data via the Glint API |
| `glint_setup()` | Configure API credentials for `read_glint_survey_api()` |
| `summarize_survey()` | Calculate comprehensive question metrics |
| `get_response_dist()` | Get response value distributions |
| `compare_cycles()` | Compare metrics across survey cycles |
| `get_correlations()` | Calculate inter-question correlations |
| `extract_survey_factors()` | Perform factor analysis |
| `analyze_attrition()` | Link survey responses to turnover |
| `analyze_by_attributes()` | Aggregate results by attribute groups |
| `pivot_long()` | Reshape data to long format |
| `aggregate_by_manager()` | Roll up results by manager |


## Dependencies

**Required:**
- dplyr (>= 1.0.0)
- tidyr (>= 1.0.0)
- readr (>= 2.0.0)
- stringr (>= 1.4.0)
- lubridate (>= 1.7.0)
- purrr (>= 0.3.0)
- httr (>= 1.4.0)

**Optional:**
- psych (>= 2.0.0) - for factor analysis

## Documentation

Access function documentation within R or on MSLearn:

```r
?read_glint_survey
?summarize_survey
?analyze_attrition
# etc.
```

Additional documentation files:
- `FUNCTION_OUTPUTS.md` - Detailed output structure documentation
- `USER_GUIDE.md` - Comprehensive user guide with examples



## Data Privacy & Security

By default, this package processes survey data locally and does not transmit data to external services. If you use the API import functions (e.g., `read_glint_survey_api()`), the package connects to Microsoft Graph to export and download survey data. All analysis is still performed locally. Ensure compliance with your organization's data handling policies when working with employee survey data.

## Code of Conduct

Please read the [Microsoft Open Source Code of Conduct](https://opensource.microsoft.com/codeofconduct/) prior to engaging with this package.

This project may contain trademarks or logos for projects, products, or services. Authorized use of Microsoft trademarks or logos is subject to and must follow Microsoft's Trademark & Brand Guidelines. Use of Microsoft trademarks or logos in modified versions of this project must not cause confusion or imply Microsoft sponsorship. Any use of third-party trademarks or logos are subject to those third-party's policies.

## Support  

This project uses GitHub Issues to track bugs and feature requests. Please search the existing 
issues before filing new issues to avoid duplicates.  For new issues, file your bug or 
feature request as a new Issue.

## Trademarks

This project may contain trademarks or logos for projects, products, or services. Authorized use of Microsoft trademarks or logos is subject to and must follow Microsoft’s Trademark & Brand Guidelines. Use of Microsoft trademarks or logos in modified versions of this project must not cause confusion or imply Microsoft sponsorship. Any use of third-party trademarks or logos are subject to those third-party’s policies.

---

**Version:** 0.1.0
**Last Updated:** March 2026

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----eval=FALSE---------------------------------------------------------------
# devtools::install_github("microsoft/vivaglint")
# library(vivaglint)

## ----eval=FALSE---------------------------------------------------------------
# survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
# survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")

## ----eval=FALSE---------------------------------------------------------------
# glint_setup(
#   tenant_id = "your-tenant-id",
#   client_id = "your-client-id",
#   client_secret = "your-client-secret",
#   experience_name = "your-experience-name"
# )
# survey <- read_glint_survey_api(
#   survey_uuid = "your-survey-uuid",
#   cycle_id = "your-cycle-id",
#   emp_id_col = "EMP ID"
# )

## ----eval=FALSE---------------------------------------------------------------
# summary <- summarize_survey(survey, scale_points = 5)

## ----eval=FALSE---------------------------------------------------------------
# # Engagement-specific questions
# engagement_qs <- c(
#   "I would recommend my team as a great place to work",
#   "My work is meaningful"
# )
# engagement_summary <- summarize_survey(survey,
#                                        scale_points = 5,
#                                        questions = engagement_qs)

## ----eval=FALSE---------------------------------------------------------------
# distributions <- get_response_dist(survey, scale_points = 5)

## ----eval=FALSE---------------------------------------------------------------
# survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
# survey_q1 <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
# survey_q2 <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
# survey_q3 <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
# 
# trends <- compare_cycles(
#   survey_q1, survey_q2, survey_q3,
#   scale_points = 5,
#   cycle_names = c("Q1 FY25", "Q2 FY25", "Q3 FY25")
# )

## ----eval=FALSE---------------------------------------------------------------
# # Direct reports only
# manager_summary <- aggregate_by_manager(survey, scale_points = 5)
# 
# # Full org tree (includes indirect reports)
# manager_full <- aggregate_by_manager(survey, scale_points = 5, full_tree = TRUE)

## ----eval=FALSE---------------------------------------------------------------
# library(dplyr)
# 
# # Managers where fewer than 50% of their team is favorable on a key item
# low_engagement_managers <- manager_summary %>%
#   filter(question == "I would recommend my team as a great place to work",
#          pct_favorable < 50) %>%
#   arrange(pct_favorable)

## ----eval=FALSE---------------------------------------------------------------
# attr_path <- system.file("extdata", "employee_attributes.csv", package = "vivaglint")
# demo_results <- analyze_by_attributes(
#   survey,
#   attribute_file = attr_path,
#   scale_points = 5,
#   attribute_cols = c("Department", "Gender", "Tenure Group"),
#   min_group_size = 10  # Suppress groups below this size for privacy
# )

## ----eval=FALSE---------------------------------------------------------------
# # Join once
# attr_path <- system.file("extdata", "employee_attributes.csv", package = "vivaglint")
# survey_enriched <- join_attributes(survey, attr_path)
# 
# # Reuse for multiple analyses — no need to re-read the file
# dept_results <- analyze_by_attributes(
#   survey_enriched,
#   scale_points = 5,
#   attribute_cols = "Department"
# )
# 
# gender_results <- analyze_by_attributes(
#   survey_enriched,
#   scale_points = 5,
#   attribute_cols = "Gender"
# )
# 
# # Filter to a subpopulation before analyzing
# na_only <- survey_enriched
# na_only$data <- filter(survey_enriched$data, Region == "North America")
# na_results <- analyze_by_attributes(na_only, scale_points = 5,
#                                     attribute_cols = "Department")

## ----eval=FALSE---------------------------------------------------------------
# # Basic attrition analysis (90, 180, and 365 days post-survey)
# attrition_path <- system.file("extdata", "attrition.csv", package = "vivaglint")
# attrition <- analyze_attrition(
#   survey,
#   attrition_file = attrition_path,
#   emp_id_col = "EMP ID",
#   term_date_col = "Termination Date",
#   scale_points = 5
# )

## ----eval=FALSE---------------------------------------------------------------
# attr_path <- system.file("extdata", "employee_attributes.csv", package = "vivaglint")
# attrition_path <- system.file("extdata", "attrition.csv", package = "vivaglint")
# survey_enriched <- join_attributes(survey, attr_path)
# 
# attrition_by_dept <- analyze_attrition(
#   survey_enriched,
#   attrition_file = attrition_path,
#   emp_id_col = "EMP ID",
#   term_date_col = "Termination Date",
#   scale_points = 5,
#   attribute_cols = c("Department", "Job Level"),
#   min_group_size = 10
# )

## ----eval=FALSE---------------------------------------------------------------
# # Pearson correlations in long format (default)
# correlations <- get_correlations(survey)
# 
# # Spearman correlations (more robust for ordinal scale data)
# correlations_spearman <- get_correlations(survey, method = "spearman")
# 
# # Correlation matrix
# cor_matrix <- get_correlations(survey, format = "matrix")

## ----eval=FALSE---------------------------------------------------------------
# # Requires the psych package
# factors <- extract_survey_factors(survey, n_factors = 3, rotation = "oblimin")
# 
# # Consolidated summary: item, factor assignment, loading, label, communality
# print(factors$factor_summary)
# 
# # Filter to items with strong factor loadings only
# strong_loaders <- dplyr::filter(factors$factor_summary, loading_label == "Strong")
# 
# # Access the raw psych object for advanced use
# factors$fa_object

## ----eval=FALSE---------------------------------------------------------------
# # Fuzzy search (default) — tolerates minor spelling differences
# flexibility_comments <- search_comments(survey, "flexibility")
# 
# # Exact, case-sensitive match
# exact_results <- search_comments(survey, "work from home", exact = TRUE)
# 
# # Broaden fuzzy tolerance to catch more spelling variation
# results <- search_comments(survey, "colaboration", max_distance = 0.3)

## ----eval=FALSE---------------------------------------------------------------
# # All responses in long format
# long_all <- pivot_long(survey, data_type = "all")
# 
# # Comments only
# long_comments <- pivot_long(survey, data_type = "comments")
# 
# # Both as separate tibbles
# both <- pivot_long(survey, data_type = "both")
# comments_df <- both$comments

## ----eval=FALSE---------------------------------------------------------------
# parts <- split_survey_data(survey)
# 
# # Numeric scores only — standard respondent columns + one score column per question
# quantitative <- parts$quantitative
# 
# # Comments only — EMP ID + all _COMMENT, _COMMENT_TOPICS, _SENSITIVE_COMMENT_FLAG columns
# qualitative <- parts$qualitative
# 
# # Pass numeric data directly to vivaglint functions
# summary <- summarize_survey(parts$quantitative, scale_points = 5,
#                             emp_id_col = "EMP ID")
# 
# # Rejoin at any time using EMP ID
# full_data <- dplyr::left_join(parts$quantitative, parts$qualitative, by = "EMP ID")

## ----eval=FALSE---------------------------------------------------------------
# # Default is 5; consider 10 or higher for sensitive analyses
# attr_path <- system.file("extdata", "employee_attributes.csv", package = "vivaglint")
# demo_results <- analyze_by_attributes(
#   survey,
#   attribute_file = attr_path,
#   scale_points = 5,
#   attribute_cols = c("Department", "Gender"),
#   min_group_size = 10
# )

#' Read Viva Glint Survey Data
#'
#' Reads a Viva Glint survey export CSV file with automatic validation and parsing.
#' This function validates the file structure, parses dates, and organizes the data
#' into a structured format ready for analysis.
#' To import directly from the Viva Glint API, use
#' \code{read_glint_survey_api()}.
#'
#' @param file_path Character string specifying the path to the CSV file
#' @param emp_id_col Character string specifying the name of the employee ID
#'   column in the survey export (e.g., \code{"Employee ID"},
#'   \code{"EmployeeID"}, \code{"emp_id"}). The column name varies by
#'   organization. The value is stored in \code{survey$metadata$emp_id_col}
#'   and used automatically by downstream functions so you do not need to
#'   repeat it on every call.
#' @param first_name_col Column name for first name (default: "First Name")
#' @param last_name_col Column name for last name (default: "Last Name")
#' @param email_col Column name for email (default: "Email")
#' @param status_col Column name for status (default: "Status")
#' @param completion_date_col Column name for survey completion date
#'   (default: "Survey Cycle Completion Date")
#' @param sent_date_col Column name for survey sent date
#'   (default: "Survey Cycle Sent Date")
#' @param encoding Character string specifying file encoding (default: "UTF-8")
#'
#' @return A \code{glint_survey} object (an S3 class extending \code{list}) with
#'   two elements:
#'   \describe{
#'     \item{data}{A tibble containing all survey responses. Date columns are
#'       parsed automatically from DD-MM-YYYY HH:MM format to POSIXct.}
#'     \item{metadata}{A list with five elements: \code{standard_columns} (the
#'       eight standard Glint column names), \code{questions} (a tibble from
#'       \code{extract_questions()} with one row per question), \code{n_respondents}
#'       (total row count), \code{n_questions} (number of questions), and
#'       \code{file_path} (the path originally supplied).}
#'   }
#'
#' @export
#'
#' @examples
#' survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
#' survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
#' head(survey$data)
#' survey$metadata$questions
read_glint_survey <- function(file_path,
                              emp_id_col = NULL,
                              first_name_col = "First Name",
                              last_name_col = "Last Name",
                              email_col = "Email",
                              status_col = "Status",
                              completion_date_col = "Survey Cycle Completion Date",
                              sent_date_col = "Survey Cycle Sent Date",
                              encoding = "UTF-8") {
  if (!file.exists(file_path)) {
    stop(sprintf(
      "File not found: '%s'\nPlease check that the file path is correct.",
      file_path
    ))
  }

  data <- tryCatch(
    {
      readr::read_csv(
        file_path,
        locale = readr::locale(encoding = encoding),
        show_col_types = FALSE
      )
    },
    error = function(e) {
      stop(sprintf(
        "Error reading CSV file: %s\nPlease ensure the file is a valid CSV format.",
        e$message
      ))
    }
  )

  build_glint_survey(
    data,
    emp_id_col,
    first_name_col = first_name_col,
    last_name_col = last_name_col,
    email_col = email_col,
    status_col = status_col,
    completion_date_col = completion_date_col,
    sent_date_col = sent_date_col,
    file_path = file_path
  )
}


#' Build a glint_survey Object
#'
#' Internal helper to validate, parse, and attach metadata.
#'
#' @param data A data frame containing Viva Glint survey data
#' @param emp_id_col Character string specifying the employee ID column name
#' @param first_name_col Column name for first name
#' @param last_name_col Column name for last name
#' @param email_col Column name for email
#' @param status_col Column name for status
#' @param completion_date_col Column name for survey completion date
#' @param sent_date_col Column name for survey sent date
#' @param file_path Character string for the source file path (or NA when not
#'   loaded from disk)
#'
#' @return A \code{glint_survey} object
#'
#' @keywords internal
build_glint_survey <- function(data,
                               emp_id_col,
                               first_name_col = "First Name",
                               last_name_col = "Last Name",
                               email_col = "Email",
                               status_col = "Status",
                               completion_date_col = "Survey Cycle Completion Date",
                               sent_date_col = "Survey Cycle Sent Date",
                               file_path = NA_character_) {
  validate_glint_structure(
    data,
    emp_id_col,
    first_name_col = first_name_col,
    last_name_col = last_name_col,
    email_col = email_col,
    status_col = status_col,
    completion_date_col = completion_date_col,
    sent_date_col = sent_date_col
  )

  standard_cols <- get_standard_columns(
    emp_id_col,
    first_name_col = first_name_col,
    last_name_col = last_name_col,
    email_col = email_col,
    status_col = status_col,
    completion_date_col = completion_date_col,
    sent_date_col = sent_date_col
  )

  date_cols <- c(completion_date_col, sent_date_col)
  for (col in date_cols) {
    if (col %in% names(data)) {
      data[[col]] <- tryCatch(
        {
          lubridate::dmy_hm(data[[col]])
        },
        error = function(e) {
          stop(sprintf(
            "Error parsing date column '%s': %s\nExpected format: DD-MM-YYYY HH:MM (e.g., '26-03-2024 09:34')",
            col, e$message
          ))
        }
      )
    }
  }

  questions_df <- extract_questions(data, emp_id_col)

  standard_column_map <- list(
    first_name = first_name_col,
    last_name = last_name_col,
    email = email_col,
    status = status_col,
    emp_id = emp_id_col,
    manager_id = "Manager ID",
    completion_date = completion_date_col,
    sent_date = sent_date_col
  )

  metadata <- list(
    standard_columns = standard_cols,
    standard_column_map = standard_column_map,
    emp_id_col = emp_id_col,
    questions = questions_df,
    n_respondents = nrow(data),
    n_questions = nrow(questions_df),
    file_path = file_path
  )

  result <- list(
    data = data,
    metadata = metadata
  )

  class(result) <- c("glint_survey", "list")
  return(result)
}


#' Validate Glint Data Structure
#'
#' Internal function to validate that a data frame conforms to the expected
#' Viva Glint export structure. Throws detailed, human-readable errors if validation fails.
#'
#' @param data A data frame to validate
#' @param emp_id_col Character string specifying the employee ID column name
#' @param first_name_col Column name for first name
#' @param last_name_col Column name for last name
#' @param email_col Column name for email
#' @param status_col Column name for status
#' @param completion_date_col Column name for survey completion date
#' @param sent_date_col Column name for survey sent date
#'
#' @return NULL (invisibly) if validation passes, otherwise throws an error
#'
#' @keywords internal
validate_glint_structure <- function(data,
                                     emp_id_col,
                                     first_name_col = "First Name",
                                     last_name_col = "Last Name",
                                     email_col = "Email",
                                     status_col = "Status",
                                     completion_date_col = "Survey Cycle Completion Date",
                                     sent_date_col = "Survey Cycle Sent Date") {
  question_suffixes <- c("_COMMENT", "_COMMENT_TOPICS", "_SENSITIVE_COMMENT_FLAG")
  question_suffix_pattern <- "(_COMMENT|_COMMENT_TOPICS|_SENSITIVE_COMMENT_FLAG)$"
  normalize_name <- function(value) trimws(value, which = "right")
  resolve_base_col <- function(stem_trim, column_names) {
    exact_match <- column_names[column_names == stem_trim]
    if (length(exact_match) > 0) {
      return(exact_match[1])
    }
    normalized_names <- normalize_name(column_names)
    matches <- column_names[normalized_names == stem_trim]
    if (length(matches) > 0) {
      return(matches[1])
    }
    ""
  }
  find_suffix_cols <- function(stem_trim, column_names) {
    candidates <- column_names[grepl(question_suffix_pattern, column_names)]
    if (length(candidates) == 0) {
      return(character(0))
    }
    stems <- normalize_name(vapply(candidates, get_question_stem, character(1)))
    candidates[stems == stem_trim]
  }
  detect_question_columns <- function(column_names) {
    suffix_cols <- column_names[grepl(question_suffix_pattern, column_names)]
    if (length(suffix_cols) == 0) {
      return(character(0))
    }
    stems_trim <- unique(normalize_name(vapply(suffix_cols, get_question_stem, character(1))))
    stems_trim <- stems_trim[nzchar(stems_trim)]
    base_cols <- vapply(stems_trim, resolve_base_col, character(1), column_names = column_names)
    base_cols <- base_cols[nzchar(base_cols)]
    unique(c(base_cols, suffix_cols))
  }
  detected_columns <- setdiff(names(data), detect_question_columns(names(data)))
  detected_columns_output <- if (length(detected_columns) == 0) {
    "  - (none)"
  } else {
    paste0("  - ", detected_columns, collapse = "\n")
  }

  required_name_map <- list(
    "Employee ID" = emp_id_col
  )
  optional_name_map <- list(
    "First Name" = first_name_col,
    "Last Name" = last_name_col,
    "Email" = email_col,
    "Status" = status_col,
    "Survey Cycle Completion Date" = completion_date_col,
    "Survey Cycle Sent Date" = sent_date_col
  )
  missing_required_names <- names(required_name_map)[vapply(
    required_name_map,
    function(x) is.null(x) || is.na(x) || !nzchar(x),
    logical(1)
  )]
  if (length(missing_required_names) > 0) {
    stop(
      "Column name(s) must be provided for: ",
      paste(missing_required_names, collapse = ", "),
      ".\n\nColumns detected in the data frame:\n",
      detected_columns_output,
      "\n\nPass the correct column name arguments or rename your columns.",
      call. = FALSE
    )
  }

  required_cols <- unname(unlist(required_name_map))
  required_cols <- required_cols[!is.na(required_cols) & nzchar(required_cols)]
  missing_required_cols <- setdiff(required_cols, names(data))
  if (length(missing_required_cols) > 0) {
    stop(sprintf(
      "Missing required standard column(s): %s\n\nYour CSV file must contain all of the following required Viva Glint columns:\n%s\n\nColumns detected in the data frame:\n%s\n\nPlease ensure you are using a complete Viva Glint survey export, or pass the correct column name arguments.",
      paste0("'", missing_required_cols, "'", collapse = ", "),
      paste0("  - ", required_cols, collapse = "\n"),
      detected_columns_output
    ))
  }

  missing_optional_names <- names(optional_name_map)[vapply(
    optional_name_map,
    function(x) is.null(x) || is.na(x) || !nzchar(x),
    logical(1)
  )]
  optional_name_map_present <- optional_name_map[setdiff(names(optional_name_map), missing_optional_names)]
  optional_cols <- unname(unlist(optional_name_map_present))
  optional_cols <- optional_cols[!is.na(optional_cols) & nzchar(optional_cols)]
  missing_optional_cols <- setdiff(optional_cols, names(data))
  missing_optional_labels <- unique(c(
    missing_optional_names,
    names(optional_name_map_present)[optional_cols %in% missing_optional_cols]
  ))

  if (length(missing_optional_labels) > 0) {
    impact_map <- list(
      "First Name" = "aggregate_by_manager() manager_name output",
      "Last Name" = "aggregate_by_manager() manager_name output",
      "Survey Cycle Completion Date" = "analyze_attrition() (survey date calculations)",
      "Survey Cycle Sent Date" = "no current functions depend on it",
      "Email" = "no current functions depend on it",
      "Status" = "no current functions depend on it"
    )
    warning_lines <- c(
      "Optional standard columns are missing or unspecified. Related functionality may be limited:",
      vapply(missing_optional_labels, function(label) {
        expected <- optional_name_map[[label]]
        expected_note <- if (is.null(expected) || !nzchar(expected)) {
          "no column name provided"
        } else {
          paste0("expected '", expected, "'")
        }
        impact <- impact_map[[label]]
        impact_note <- if (is.null(impact)) "some functionality may be limited" else impact
        paste0("  - ", label, " (", expected_note, "). Affects: ", impact_note, ".")
      }, character(1)),
      "",
      "Columns detected in the data frame:",
      detected_columns_output
    )
    warning(paste(warning_lines, collapse = "\n"), call. = FALSE)
  }

  all_standard_cols <- get_standard_columns(
    emp_id_col,
    first_name_col = first_name_col,
    last_name_col = last_name_col,
    email_col = email_col,
    status_col = status_col,
    completion_date_col = completion_date_col,
    sent_date_col = sent_date_col
  )
  question_cols <- setdiff(names(data), all_standard_cols)
  suffix_cols <- question_cols[grepl(question_suffix_pattern, question_cols)]
  question_stems <- unique(normalize_name(vapply(suffix_cols, get_question_stem, character(1))))
  question_stems <- question_stems[nzchar(question_stems)]

  if (length(question_stems) == 0) {
    stop(
      "No question columns found in the data.\n\nA Viva Glint export should contain at least one question with its associated columns:\n  - [Question Text]\n  - [Question Text]_COMMENT\n  - [Question Text]_COMMENT_TOPICS\n  - [Question Text]_SENSITIVE_COMMENT_FLAG\n\nPlease check that you are using a complete survey export file."
    )
  }

  expected_suffixes <- c("", question_suffixes)
  orphaned_cols <- character(0)
  incomplete_questions <- character(0)

  for (stem in question_stems) {
    base_col <- resolve_base_col(stem, names(data))
    expected_cols <- c(
      if (nzchar(base_col)) base_col else stem,
      paste0(stem, question_suffixes)
    )
    existing_suffix_cols <- find_suffix_cols(stem, names(data))
    existing_suffix_types <- vapply(question_suffixes, function(suffix) {
      any(endsWith(existing_suffix_cols, suffix))
    }, logical(1))
    found_cols <- c(nzchar(base_col), existing_suffix_types)

    # If we have some but not all columns for this question, it's incomplete
    if (any(found_cols) && !all(found_cols)) {
      missing_for_q <- expected_cols[!found_cols]
      incomplete_questions <- c(incomplete_questions, sprintf(
        "  - Question '%s' is missing: %s",
        stem,
        paste0("'", basename(missing_for_q), "'", collapse = ", ")
      ))
    }
  }

  if (length(suffix_cols) > 0) {
    suffix_stems <- normalize_name(vapply(suffix_cols, get_question_stem, character(1)))
    base_matches <- vapply(suffix_stems, resolve_base_col, character(1), column_names = names(data))
    orphaned_cols <- unique(suffix_cols[!nzchar(base_matches)])
  }

  # Report errors if found
  error_messages <- character(0)

  if (length(incomplete_questions) > 0) {
    error_messages <- c(
      error_messages,
      sprintf(
        "Incomplete question column sets found:\n%s\n\nEach question must have all four columns:\n  - [Question Text]\n  - [Question Text]_COMMENT\n  - [Question Text]_COMMENT_TOPICS\n  - [Question Text]_SENSITIVE_COMMENT_FLAG",
        paste(incomplete_questions, collapse = "\n")
      )
    )
  }

  if (length(orphaned_cols) > 0) {
    error_messages <- c(
      error_messages,
      sprintf(
        "Orphaned columns found that don't belong to a complete question set:\n  %s\n\nPlease check your CSV export for incomplete or corrupted question columns.",
        paste0("  - '", orphaned_cols, "'", collapse = "\n")
      )
    )
  }

  if (length(error_messages) > 0) {
    stop(paste(error_messages, collapse = "\n\n"))
  }

  invisible(NULL)
}


#' Extract Questions from Glint Survey
#'
#' Parses column names to extract unique questions and their associated column names.
#'
#' @param data A data frame or glint_survey object containing survey data
#' @param emp_id_col Character string specifying the employee ID column name.
#'   When \code{data} is a \code{glint_survey} object this is resolved
#'   automatically from \code{data$metadata$emp_id_col}. Required when
#'   \code{data} is a plain data frame.
#'
#' @return A tibble with one row per question containing:
#'   \describe{
#'     \item{question}{The question text}
#'     \item{response_col}{Column name for numeric responses}
#'     \item{comment_col}{Column name for comments}
#'     \item{topics_col}{Column name for comment topics}
#'     \item{flag_col}{Column name for sensitive comment flags}
#'   }
#'
#' @export
#'
#' @examples
#' survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
#' survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
#' questions <- extract_questions(survey)
#' print(questions)
extract_questions <- function(data, emp_id_col = NULL) {
  if (inherits(data, "glint_survey")) {
    emp_id_col <- data$metadata$emp_id_col %||% emp_id_col
    standard_cols <- data$metadata$standard_columns
    data <- data$data
  } else {
    standard_cols <- NULL
  }
  if (is.null(emp_id_col)) {
    stop("emp_id_col must be specified. When loading with read_glint_survey(), pass emp_id_col to store it automatically.")
  }

  standard_cols <- standard_cols %||% get_standard_columns(emp_id_col)
  question_cols <- setdiff(names(data), standard_cols)
  question_suffixes <- c("_COMMENT", "_COMMENT_TOPICS", "_SENSITIVE_COMMENT_FLAG")
  question_suffix_pattern <- "(_COMMENT|_COMMENT_TOPICS|_SENSITIVE_COMMENT_FLAG)$"
  normalize_name <- function(value) trimws(value, which = "right")
  resolve_base_col <- function(stem_trim, column_names) {
    exact_match <- column_names[column_names == stem_trim]
    if (length(exact_match) > 0) {
      return(exact_match[1])
    }
    normalized_names <- normalize_name(column_names)
    matches <- column_names[normalized_names == stem_trim]
    if (length(matches) > 0) {
      return(matches[1])
    }
    ""
  }
  find_suffix_col <- function(stem_trim, suffix, column_names) {
    candidates <- column_names[grepl(paste0(suffix, "$"), column_names)]
    if (length(candidates) == 0) {
      return(paste0(stem_trim, suffix))
    }
    stems <- normalize_name(vapply(candidates, get_question_stem, character(1)))
    matches <- candidates[stems == stem_trim]
    if (length(matches) > 0) {
      return(matches[1])
    }
    paste0(stem_trim, suffix)
  }
  suffix_cols <- question_cols[grepl(question_suffix_pattern, question_cols)]
  question_stems <- unique(normalize_name(vapply(suffix_cols, get_question_stem, character(1))))
  question_stems <- question_stems[nzchar(question_stems)]

  questions_df <- purrr::map_dfr(question_stems, function(stem) {
    response_col <- resolve_base_col(stem, names(data))
    if (!nzchar(response_col)) {
      response_col <- stem
    }
    dplyr::tibble(
      question = stem,
      response_col = response_col,
      comment_col = find_suffix_col(stem, "_COMMENT", names(data)),
      topics_col = find_suffix_col(stem, "_COMMENT_TOPICS", names(data)),
      flag_col = find_suffix_col(stem, "_SENSITIVE_COMMENT_FLAG", names(data))
    )
  })

  return(questions_df)
}


#' Join Employee Attributes to Survey Data
#'
#' Reads employee attribute data from a CSV file or data frame and joins it to
#' a survey object by employee ID. Returns an enriched \code{glint_survey}
#' object that can be passed directly to \code{analyze_by_attributes()} or any
#' other package function, eliminating the need to re-read and re-join the
#' attribute file on every call.
#'
#' @param survey A \code{glint_survey} object or data frame containing survey data
#' @param attribute_source Either a character string file path to a CSV file, or
#'   a data frame containing employee attributes. All columns are coerced to
#'   character to avoid type conflicts during joining.
#' @param emp_id_col Character string specifying the employee ID column name.
#'   Must match the column name in both the survey data and the attribute data.
#'   When \code{survey} is a \code{glint_survey} object this defaults to
#'   \code{survey$metadata$emp_id_col} (set at import via
#'   \code{read_glint_survey()}), so you can omit this argument if you loaded
#'   the survey with the correct \code{emp_id_col}.
#'
#' @return If \code{survey} is a \code{glint_survey} object, returns an enriched
#'   \code{glint_survey} with the attribute columns appended to \code{$data} and
#'   the names of all joined attribute columns stored in
#'   \code{$metadata$attribute_cols}. If \code{survey} is a plain data frame,
#'   returns the joined data frame.
#'
#'   Respondents with no match in the attribute data will have \code{NA} for all
#'   attribute columns; a message is emitted indicating how many were affected.
#'
#' @export
#'
#' @examples
#' survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
#' attr_path <- system.file("extdata", "employee_attributes.csv", package = "vivaglint")
#' survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
#'
#' survey_enriched <- join_attributes(survey, attr_path, emp_id_col = "EMP ID")
#' results <- analyze_by_attributes(survey_enriched, scale_points = 5,
#'                                  attribute_cols = "Department")
join_attributes <- function(survey, attribute_source, emp_id_col = NULL) {
  if (inherits(survey, "glint_survey")) {
    emp_id_col <- emp_id_col %||% survey$metadata$emp_id_col
    data <- survey$data
  } else if (is.data.frame(survey)) {
    data <- survey
  } else {
    stop("survey must be a glint_survey object or a data frame")
  }

  if (is.null(emp_id_col)) {
    stop("emp_id_col must be specified when survey is a plain data frame (it cannot be resolved from metadata)")
  }

  if (!emp_id_col %in% names(data)) {
    stop(sprintf("Column '%s' not found in survey data", emp_id_col))
  }

  if (is.character(attribute_source) && length(attribute_source) == 1) {
    if (!file.exists(attribute_source)) {
      stop(sprintf("Attribute file not found: '%s'", attribute_source))
    }
    attributes <- readr::read_csv(
      attribute_source,
      show_col_types = FALSE,
      col_types = readr::cols(.default = readr::col_character())
    )
  } else if (is.data.frame(attribute_source)) {
    attributes <- dplyr::mutate(
      attribute_source,
      dplyr::across(dplyr::everything(), as.character)
    )
  } else {
    stop("attribute_source must be a file path (character string) or a data frame")
  }

  if (!emp_id_col %in% names(attributes)) {
    stop(sprintf("Column '%s' not found in attribute data", emp_id_col))
  }

  new_attr_cols <- setdiff(names(attributes), emp_id_col)

  overlap <- intersect(new_attr_cols, names(data))
  if (length(overlap) > 0) {
    warning(sprintf(
      "The following columns already exist in the survey data and will be overwritten: %s",
      paste0("'", overlap, "'", collapse = ", ")
    ))
    data <- dplyr::select(data, -dplyr::all_of(overlap))
  }

  joined_data <- dplyr::left_join(
    data,
    attributes,
    by = emp_id_col
  )

  n_unmatched <- sum(!data[[emp_id_col]] %in% attributes[[emp_id_col]])
  if (n_unmatched > 0) {
    message(sprintf(
      "%d respondent(s) had no match in the attribute data and will have NA for all attribute columns.",
      n_unmatched
    ))
  }

  if (inherits(survey, "glint_survey")) {
    existing_attr_cols <- survey$metadata$attribute_cols %||% character(0)
    survey$data <- joined_data
    survey$metadata$attribute_cols <- unique(c(existing_attr_cols, new_attr_cols))
    return(survey)
  } else {
    return(joined_data)
  }
}

# Internal ggplot2 plot helpers for plot = TRUE on analysis functions.
# None of these functions are exported; all names are prefixed with .plot_.

.check_ggplot2 <- function() {
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop(
      "Package 'ggplot2' is required for plotting. ",
      "Install it with: install.packages('ggplot2')"
    )
  }
}


# ── summarize_survey ──────────────────────────────────────────────────────────

.plot_survey_summary <- function(results) {
  long <- tidyr::pivot_longer(
    results,
    cols      = c("pct_unfavorable", "pct_neutral", "pct_favorable"),
    names_to  = "sentiment",
    values_to = "pct"
  )
  long$sentiment <- factor(
    long$sentiment,
    levels = c("pct_unfavorable", "pct_neutral", "pct_favorable"),
    labels = c("Unfavorable", "Neutral", "Favorable")
  )
  gs            <- stats::setNames(results$glint_score, results$question)
  long$question <- factor(long$question,
    levels = results$question[order(results$glint_score)])

  ggplot2::ggplot(long,
    ggplot2::aes(.data$question, .data$pct, fill = .data$sentiment)) +
    ggplot2::geom_col(width = 0.65,
      position = ggplot2::position_stack(reverse = TRUE)) +
    ggplot2::geom_text(
      data = long[long$sentiment == "Favorable", ],
      ggplot2::aes(y = 103,
        label = gs[as.character(.data$question)]),
      hjust = 0, size = 3.2, fontface = "bold", color = "#333333"
    ) +
    ggplot2::scale_fill_manual(
      values = c(Favorable = "#0078D4", Neutral = "#B0BEC5",
                 Unfavorable = "#C42B1C"),
      breaks = c("Favorable", "Neutral", "Unfavorable")
    ) +
    ggplot2::scale_y_continuous(
      limits = c(0, 115), breaks = seq(0, 100, 25),
      labels = function(x) paste0(x, "%")
    ) +
    ggplot2::coord_flip() +
    ggplot2::labs(
      title    = "Favorability Breakdown",
      subtitle = "Questions sorted by Glint Score (0-100); score shown at right",
      x = NULL, y = "% of Respondents", fill = NULL
    ) +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      plot.title           = ggplot2::element_text(face = "bold"),
      legend.position      = "bottom",
      panel.grid.major.y   = ggplot2::element_blank()
    )
}


# ── get_response_dist ─────────────────────────────────────────────────────────

.plot_response_dist <- function(results) {
  pct_cols <- grep("^pct_", names(results), value = TRUE)
  values   <- sort(as.numeric(sub("^pct_", "", pct_cols)))
  n_vals   <- length(values)

  # Gradient: dark red → pink → grey → light blue → dark blue
  pal        <- grDevices::colorRampPalette(
    c("#C42B1C", "#F4ACAC", "#B0BEC5", "#6FB3E0", "#0078D4")
  )(n_vals)
  names(pal) <- as.character(values)

  long <- tidyr::pivot_longer(
    results,
    cols      = dplyr::all_of(pct_cols),
    names_to  = "value",
    values_to = "pct"
  )
  long$value <- factor(sub("^pct_", "", long$value),
    levels = as.character(values))

  # Sort questions by weighted mean response (lowest score at bottom)
  wmeans    <- tapply(
    as.numeric(as.character(long$value)) * long$pct,
    long$question, sum, na.rm = TRUE
  ) / tapply(long$pct, long$question, sum, na.rm = TRUE)
  long$question <- factor(long$question, levels = names(sort(wmeans)))

  ggplot2::ggplot(long,
    ggplot2::aes(.data$question, .data$pct, fill = .data$value)) +
    ggplot2::geom_col(width = 0.65,
      position = ggplot2::position_stack(reverse = TRUE)) +
    ggplot2::scale_fill_manual(values = pal) +
    ggplot2::scale_y_continuous(
      labels = function(x) paste0(x, "%")
    ) +
    ggplot2::coord_flip() +
    ggplot2::labs(
      title    = "Response Value Distribution",
      subtitle = "Each segment = % of respondents selecting that scale value",
      x = NULL, y = "% of Respondents", fill = "Value"
    ) +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      plot.title           = ggplot2::element_text(face = "bold"),
      legend.position      = "bottom",
      panel.grid.major.y   = ggplot2::element_blank()
    ) +
    ggplot2::guides(fill = ggplot2::guide_legend(nrow = 1))
}


# ── compare_cycles ────────────────────────────────────────────────────────────

.plot_compare_cycles <- function(results, cycle_names) {
  results$cycle <- factor(results$cycle, levels = cycle_names)
  last_cycle    <- cycle_names[length(cycle_names)]
  labels_df     <- results[results$cycle == last_cycle, ]

  score_min <- max(0,   min(results$glint_score, na.rm = TRUE) - 10)
  score_max <- min(100, max(results$glint_score, na.rm = TRUE) + 15)

  ggplot2::ggplot(results,
    ggplot2::aes(.data$cycle, .data$glint_score,
      color = .data$question, group = .data$question)) +
    ggplot2::geom_line(linewidth = 0.9) +
    ggplot2::geom_point(size = 2.5) +
    ggplot2::geom_text(
      data = labels_df,
      ggplot2::aes(label = .data$glint_score),
      hjust = -0.7, size = 3, show.legend = FALSE
    ) +
    ggplot2::scale_y_continuous(
      limits = c(score_min, score_max),
      breaks = seq(0, 100, 10)
    ) +
    ggplot2::scale_color_brewer(palette = "Dark2") +
    ggplot2::labs(
      title    = "Glint Score Trends",
      subtitle = "Score on 0-100 scale; score value shown at final cycle",
      x = NULL, y = "Glint Score", color = NULL
    ) +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      plot.title        = ggplot2::element_text(face = "bold"),
      legend.position   = "right",
      panel.grid.minor  = ggplot2::element_blank()
    )
}


# ── get_correlations ──────────────────────────────────────────────────────────

.plot_correlations <- function(results) {
  questions      <- unique(results$question1)
  results$question1 <- factor(results$question1, levels = questions)
  results$question2 <- factor(results$question2, levels = rev(questions))

  ggplot2::ggplot(results,
    ggplot2::aes(.data$question1, .data$question2,
      fill = .data$correlation)) +
    ggplot2::geom_tile(color = "white", linewidth = 0.5) +
    ggplot2::geom_text(
      ggplot2::aes(label = sprintf("%.2f", .data$correlation)),
      size  = 3.2,
      color = ifelse(abs(results$correlation) > 0.5, "white", "#333333")
    ) +
    ggplot2::scale_fill_gradient2(
      low = "#C42B1C", mid = "#FFFFFF", high = "#0078D4",
      midpoint = 0, limits = c(-1, 1), name = "r"
    ) +
    ggplot2::labs(
      title    = "Question Correlation Heatmap",
      subtitle = "Pairwise complete observations",
      x = NULL, y = NULL
    ) +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      plot.title  = ggplot2::element_text(face = "bold"),
      axis.text.x = ggplot2::element_text(angle = 35, hjust = 1),
      panel.grid  = ggplot2::element_blank()
    )
}


# ── extract_survey_factors ────────────────────────────────────────────────────

.plot_survey_factors <- function(factor_summary) {
  factor_summary$question <- factor(
    factor_summary$question,
    levels = rev(unique(factor_summary$question))
  )

  ggplot2::ggplot(factor_summary,
    ggplot2::aes(.data$factor, .data$question,
      fill = .data$loading)) +
    ggplot2::geom_tile(color = "white", linewidth = 0.8) +
    ggplot2::geom_text(
      ggplot2::aes(label = sprintf("%.2f", .data$loading)),
      size  = 3.5,
      color = ifelse(factor_summary$loading > 0.55, "white", "#333333")
    ) +
    ggplot2::scale_fill_gradient(
      low = "#D6ECFA", high = "#0078D4", limits = c(0, 1),
      name = "Loading"
    ) +
    ggplot2::labs(
      title    = "Factor Loading Heatmap",
      subtitle = "Cells shown where |loading| meets minimum threshold",
      x = NULL, y = NULL
    ) +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      plot.title  = ggplot2::element_text(face = "bold"),
      panel.grid  = ggplot2::element_blank(),
      axis.text.x = ggplot2::element_text(size = 11)
    )
}


# ── analyze_attrition ─────────────────────────────────────────────────────────

.plot_attrition <- function(results, attribute_cols) {
  id_cols   <- c(attribute_cols, "question", "days")
  id_cols   <- id_cols[id_cols %in% names(results)]
  plot_data <- tidyr::pivot_longer(
    results[, c(id_cols, "favorable_attrition", "unfavorable_attrition"),
            drop = FALSE],
    cols      = c("favorable_attrition", "unfavorable_attrition"),
    names_to  = "group",
    values_to = "attrition"
  )
  plot_data$group <- factor(
    plot_data$group,
    levels = c("favorable_attrition", "unfavorable_attrition"),
    labels = c("Favorable", "Unfavorable")
  )
  plot_data$days <- factor(
    paste0(plot_data$days, " days"),
    levels = paste0(sort(unique(results$days)), " days")
  )

  p <- ggplot2::ggplot(plot_data,
      ggplot2::aes(.data$question, .data$attrition, fill = .data$group)) +
    ggplot2::geom_col(position = "dodge", width = 0.65) +
    ggplot2::scale_fill_manual(
      values = c(Favorable = "#0078D4", Unfavorable = "#C42B1C")
    ) +
    ggplot2::scale_y_continuous(
      labels = function(x) paste0(round(x * 100, 1), "%")
    ) +
    ggplot2::labs(
      title    = "Attrition Rate by Favorability",
      subtitle = paste0("Proportion who left within each window; ",
        "favorable vs. unfavorable respondents"),
      x = NULL, y = "Attrition Rate", fill = NULL
    ) +
    ggplot2::theme_minimal(base_size = 11) +
    ggplot2::theme(
      plot.title          = ggplot2::element_text(face = "bold"),
      legend.position     = "bottom",
      axis.text.x         = ggplot2::element_text(angle = 35, hjust = 1),
      panel.grid.major.x  = ggplot2::element_blank(),
      strip.text          = ggplot2::element_text(face = "bold")
    )

  attr_col <- if (!is.null(attribute_cols)) attribute_cols[1] else NULL

  if (!is.null(attr_col) && attr_col %in% names(plot_data)) {
    p <- p + ggplot2::facet_grid(
      stats::reformulate("days", attr_col)
    )
  } else {
    p <- p + ggplot2::facet_wrap(~days, nrow = 1)
  }

  p
}


# ── analyze_by_attributes ─────────────────────────────────────────────────────

.plot_by_attributes <- function(results, attribute_cols) {
  attr_col <- attribute_cols[1]

  if (length(attribute_cols) > 1) {
    message(
      "plot = TRUE shows results for the first attribute column ('",
      attr_col, "') only."
    )
    results <- results %>%
      dplyr::group_by(
        dplyr::across(dplyr::all_of(c(attr_col, "question")))
      ) %>%
      dplyr::summarise(
        glint_score = stats::weighted.mean(
          .data$glint_score, .data$group_size, na.rm = TRUE
        ),
        group_size = sum(.data$group_size, na.rm = TRUE),
        .groups = "drop"
      )
  }

  score_min <- max(0,   min(results$glint_score, na.rm = TRUE) - 10)
  score_max <- min(100, max(results$glint_score, na.rm = TRUE) + 15)

  ggplot2::ggplot(results,
    ggplot2::aes(.data$glint_score, .data[[attr_col]])) +
    ggplot2::geom_point(
      ggplot2::aes(size = .data$group_size, color = .data$glint_score)
    ) +
    ggplot2::geom_text(
      ggplot2::aes(label = round(.data$glint_score)),
      hjust = -1.0, size = 2.8
    ) +
    ggplot2::scale_color_gradient(
      low = "#C42B1C", high = "#0078D4",
      limits = c(score_min, score_max), name = "Score"
    ) +
    ggplot2::scale_size_continuous(range = c(2, 6), name = "Group n") +
    ggplot2::scale_x_continuous(limits = c(score_min, score_max + 5)) +
    ggplot2::facet_wrap(~question, nrow = 2) +
    ggplot2::labs(
      title    = paste0("Glint Score by ", attr_col),
      subtitle = "Dot size = group respondent count",
      x = "Glint Score", y = NULL
    ) +
    ggplot2::theme_minimal(base_size = 11) +
    ggplot2::theme(
      plot.title          = ggplot2::element_text(face = "bold"),
      panel.grid.major.y  = ggplot2::element_blank(),
      strip.text          = ggplot2::element_text(face = "bold", size = 9),
      legend.position     = "bottom"
    )
}


# ── aggregate_by_manager ─────────────────────────────────────────────────────

.plot_manager <- function(results) {
  n_questions <- length(unique(results$question))

  # Sort managers by mean Glint Score across all questions
  mgr_order         <- tapply(results$glint_score, results$manager_name,
    mean, na.rm = TRUE)
  results$manager_name <- factor(results$manager_name,
    levels = names(sort(mgr_order)))

  score_min  <- max(0,   min(results$glint_score, na.rm = TRUE) - 10)
  score_max  <- min(100, max(results$glint_score, na.rm = TRUE) + 15)

  # Per-question average for the reference line
  pop_avg_df <- results %>%
    dplyr::group_by(.data$question) %>%
    dplyr::summarise(
      pop_avg = mean(.data$glint_score, na.rm = TRUE),
      .groups = "drop"
    )

  p <- ggplot2::ggplot(results,
      ggplot2::aes(.data$glint_score, .data$manager_name)) +
    ggplot2::geom_segment(
      ggplot2::aes(
        x    = score_min,
        xend = .data$glint_score,
        y    = .data$manager_name,
        yend = .data$manager_name
      ),
      color = "#B0BEC5", linewidth = 0.8
    ) +
    ggplot2::geom_vline(
      data     = pop_avg_df,
      ggplot2::aes(xintercept = .data$pop_avg),
      linetype = "dashed", color = "#555555", linewidth = 0.5
    ) +
    ggplot2::geom_point(
      ggplot2::aes(size = .data$team_size, color = .data$glint_score)
    ) +
    ggplot2::geom_text(
      ggplot2::aes(label = .data$glint_score),
      hjust = -1.1, size = 3.2, fontface = "bold"
    ) +
    ggplot2::scale_color_gradient(
      low = "#C42B1C", high = "#0078D4",
      limits = c(score_min, score_max), name = "Score"
    ) +
    ggplot2::scale_size_continuous(range = c(3, 9), name = "Team n") +
    ggplot2::scale_x_continuous(limits = c(score_min, score_max + 5)) +
    ggplot2::labs(
      title    = "Team Glint Scores by Manager",
      subtitle = paste0("Sorted by score; dashed line = group mean; ",
        "dot size = team respondent count"),
      x = "Glint Score", y = NULL
    ) +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      plot.title          = ggplot2::element_text(face = "bold"),
      panel.grid.major.y  = ggplot2::element_blank(),
      legend.position     = "bottom"
    )

  if (n_questions > 1) {
    p <- p + ggplot2::facet_wrap(~question)
  }

  p
}

#' Pivot Survey Data to Long Format
#'
#' Transforms survey data from wide format (one row per respondent) to long format
#' (one row per respondent-question combination). Can return all responses, only
#' comments, or both depending on the data_type parameter.
#'
#' @param survey A glint_survey object or data frame containing survey data
#' @param data_type Character string indicating what data to return: "all" for all
#'   responses including those without comments, "comments" for only responses with
#'   comments, or "both" for separate tibbles (default: "all")
#' @param include_empty Logical indicating whether to include empty comments when
#'   data_type = "comments" (default: FALSE)
#' @param include_standard_cols Logical indicating whether to include standard
#'   columns (EMP ID, Manager ID, etc.) in the output (default: TRUE)
#'
#' @return When \code{data_type = "all"} or \code{"comments"}, a single tibble
#'   in long format with columns:
#'   \describe{
#'     \item{Standard columns}{EMP ID, First Name, Last Name, etc. (if
#'       \code{include_standard_cols = TRUE})}
#'     \item{question}{The question text}
#'     \item{response}{Numeric response value}
#'     \item{comment}{Comment text (NA if not provided)}
#'     \item{comment_topics}{Comma-separated topic tags (NA if not provided)}
#'     \item{sensitive_flag}{Sensitivity flag value}
#'   }
#'   When \code{data_type = "both"}, a named list with two elements:
#'   \describe{
#'     \item{all}{Long-format tibble of all responses (same structure as above)}
#'     \item{comments}{Long-format tibble filtered to rows with non-empty comments}
#'   }
#'
#' @export
#'
#' @examples
#' survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
#' survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
#'
#' # Get all responses (with and without comments)
#' all_data <- pivot_long(survey)
#'
#' # Get only responses with comments
#' comments_only <- pivot_long(survey, data_type = "comments")
#'
#' # Get both separately
#' both <- pivot_long(survey, data_type = "both")
#' all_responses <- both$all
#' comments <- both$comments
pivot_long <- function(survey, data_type = "all", include_empty = FALSE, include_standard_cols = TRUE) {
  if (!data_type %in% c("all", "comments", "both")) {
    stop("data_type must be one of: 'all', 'comments', or 'both'")
  }

  if (inherits(survey, "glint_survey")) {
    emp_id_col <- survey$metadata$emp_id_col
    data <- survey$data
    questions <- survey$metadata$questions
  } else {
    emp_id_col <- NULL
    data <- survey
    questions <- extract_questions(data)
  }
  if (is.null(emp_id_col)) {
    stop("emp_id_col could not be determined. Load your survey with read_glint_survey() and specify emp_id_col.")
  }

  standard_cols <- if (inherits(survey, "glint_survey")) {
    survey$metadata$standard_columns %||% get_standard_columns(emp_id_col)
  } else {
    get_standard_columns(emp_id_col)
  }
  present_standard_cols <- intersect(standard_cols, names(data))
  standard_data <- data[, present_standard_cols, drop = FALSE]

  long_data <- purrr::map_dfr(seq_len(nrow(questions)), function(i) {
    q <- questions[i, ]

    question_data <- dplyr::tibble(
      question = q$question,
      response = data[[q$response_col]],
      comment = data[[q$comment_col]],
      comment_topics = data[[q$topics_col]],
      sensitive_flag = data[[q$flag_col]]
    )

    if (include_standard_cols) {
      question_data <- dplyr::bind_cols(standard_data, question_data)
    }

    return(question_data)
  })

  if (data_type == "all") {
    return(long_data)
  } else if (data_type == "comments") {
    if (!include_empty) {
      long_data <- long_data %>%
        dplyr::filter(!is.na(comment) & comment != "")
    }
    return(long_data)
  } else {  # data_type == "both"
    comments_data <- long_data
    if (!include_empty) {
      comments_data <- comments_data %>%
        dplyr::filter(!is.na(comment) & comment != "")
    }

    return(list(
      all = long_data,
      comments = comments_data
    ))
  }
}

#' Get Question Stem from Column Name
#'
#' Extracts the base question text from a column name by removing
#' standard suffixes (_COMMENT, _COMMENT_TOPICS, _SENSITIVE_COMMENT_FLAG).
#'
#' @param col_name Character string representing a column name
#'
#' @return Character string with the base question text
#'
#' @keywords internal
get_question_stem <- function(col_name) {
  # Remove the standard suffixes
  col_name <- stringr::str_remove(col_name, "_COMMENT_TOPICS$")
  col_name <- stringr::str_remove(col_name, "_SENSITIVE_COMMENT_FLAG$")
  col_name <- stringr::str_remove(col_name, "_COMMENT$")

  return(col_name)
}


#' Parse Comment Topics
#'
#' Splits comma-separated topic strings into vectors, handling empty and NA values.
#'
#' @param topics Character vector of comma-separated topic strings
#' @param return_format Character string specifying output format: "list" (default) or "tidy"
#'
#' @return If return_format is "list", returns a list of character vectors.
#'   If return_format is "tidy", returns a tibble with columns for original index and individual topics.
#'
#' @keywords internal
parse_comment_topics <- function(topics, return_format = "list") {
  if (!return_format %in% c("list", "tidy")) {
    stop("return_format must be either 'list' or 'tidy'")
  }

  topics <- ifelse(is.na(topics) | topics == "", NA_character_, topics)

  # Split by comma and trim whitespace
  topic_list <- lapply(topics, function(x) {
    if (is.na(x)) {
      return(character(0))
    }
    stringr::str_trim(stringr::str_split(x, ",")[[1]])
  })

  if (return_format == "list") {
    return(topic_list)
  } else {
    # Return tidy format: one row per topic
    result <- purrr::map2_dfr(
      seq_along(topic_list),
      topic_list,
      function(idx, topics) {
        if (length(topics) == 0) {
          return(NULL)
        }
        dplyr::tibble(
          index = idx,
          topic = topics
        )
      }
    )
    return(result)
  }
}


#' Convert Raw Mean to Glint Score
#'
#' Transforms a raw mean response on a rating scale to the 100-point Glint Score
#' as displayed in the Viva Glint UI. Rounded to the nearest whole number, consistent
#' with Glint's rounding convention.
#'
#' @param mean_val Numeric raw mean value
#' @param scale_points Integer number of scale points (2-11)
#'
#' @return Numeric Glint Score on a 0-100 scale
#'
#' @keywords internal
mean_to_glint_score <- function(mean_val, scale_points) {
  round(((mean_val - 1) / (scale_points - 1)) * 100)
}


#' Get Standard Column Names
#'
#' Returns the vector of standard column names expected in Viva Glint exports.
#' The employee ID column name is configurable because customers use different
#' names. "Manager ID" is included for question-detection exclusion but is not
#' required to be present.
#'
#' @param emp_id_col Character string specifying the employee ID column name
#' @param first_name_col Column name for first name (default: "First Name")
#' @param last_name_col Column name for last name (default: "Last Name")
#' @param email_col Column name for email (default: "Email")
#' @param status_col Column name for status (default: "Status")
#' @param completion_date_col Column name for survey completion date
#'   (default: "Survey Cycle Completion Date")
#' @param sent_date_col Column name for survey sent date
#'   (default: "Survey Cycle Sent Date")
#'
#' @return Character vector of standard column names
#'
#' @keywords internal
get_standard_columns <- function(emp_id_col,
                                 first_name_col = "First Name",
                                 last_name_col = "Last Name",
                                 email_col = "Email",
                                 status_col = "Status",
                                 completion_date_col = "Survey Cycle Completion Date",
                                 sent_date_col = "Survey Cycle Sent Date") {
  cols <- c(first_name_col, last_name_col, email_col, status_col, emp_id_col, "Manager ID",
            completion_date_col, sent_date_col)
  cols <- cols[!is.na(cols)]
  cols[nzchar(cols)]
}


#' Get Favorability Map for Scale Points
#'
#' Returns the favorability classification for a given scale point count,
#' based on Glint standards.
#'
#' @param scale_points Integer specifying the number of scale points (2-11)
#'
#' @return A list with three elements: favorable, neutral, and unfavorable,
#'   each containing a vector of scale points classified into that category
#'
#' @keywords internal
get_favorability_map <- function(scale_points) {
  favorability_map <- list(
    "2" = list(favorable = c(2), neutral = c(), unfavorable = c(1)),
    "3" = list(favorable = c(3), neutral = c(2), unfavorable = c(1)),
    "4" = list(favorable = c(4), neutral = c(2, 3), unfavorable = c(1)),
    "5" = list(favorable = c(4, 5), neutral = c(3), unfavorable = c(1, 2)),
    "6" = list(favorable = c(4, 5, 6), neutral = c(), unfavorable = c(1, 2, 3)),
    "7" = list(favorable = c(6, 7), neutral = c(4, 5), unfavorable = c(1, 2, 3)),
    "8" = list(favorable = c(6, 7, 8), neutral = c(4, 5), unfavorable = c(1, 2, 3)),
    "9" = list(favorable = c(7, 8, 9), neutral = c(4, 5, 6), unfavorable = c(1, 2, 3)),
    "10" = list(favorable = c(8, 9, 10), neutral = c(4, 5, 6, 7), unfavorable = c(1, 2, 3)),
    "11" = list(favorable = c(10, 11), neutral = c(8, 9), unfavorable = c(1, 2, 3, 4, 5, 6, 7))
  )

  return(favorability_map[[as.character(scale_points)]])
}

#' Summarize Survey Questions
#'
#' Calculates comprehensive metrics for survey questions, returning a summary
#' analysis in tidy format. This includes mean, standard deviation, Glint Score,
#' response counts, skip counts, and favorability percentages.
#'
#' @param survey A glint_survey object or data frame containing survey data
#' @param scale_points Integer specifying the number of scale points (2-11)
#' @param questions Character vector of question text(s) to analyze, or "all" to
#'   analyze all questions (default: "all")
#' @param emp_id_col Character string specifying the employee ID column name
#' @param plot Logical. If \code{TRUE}, prints a favorability stacked bar chart
#'   sorted by Glint Score and returns the data invisibly. Requires
#'   \pkg{ggplot2}. Default: \code{FALSE}.
#'
#' @return A tibble with one row per question containing:
#'   \describe{
#'     \item{question}{The question text}
#'     \item{mean}{Mean of numeric responses (raw scale)}
#'     \item{sd}{Standard deviation of numeric responses}
#'     \item{glint_score}{Mean transformed to a 0-100 scale, matching the score
#'       displayed in the Viva Glint UI: \code{round(((mean - 1) / (scale_points - 1)) * 100)}}
#'     \item{n_responses}{Count of non-blank, non-null responses}
#'     \item{n_skips}{Count of blank or null responses}
#'     \item{n_total}{Total number of respondents}
#'     \item{pct_favorable}{Percentage of responses classified as favorable}
#'     \item{pct_neutral}{Percentage of responses classified as neutral}
#'     \item{pct_unfavorable}{Percentage of responses classified as unfavorable}
#'   }
#'   When \code{plot = TRUE}, the same tibble is returned invisibly after
#'   printing the plot.
#'
#' @export
#'
#' @examples
#' survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
#' survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
#'
#' # Summarize all questions (5-point scale)
#' summary <- summarize_survey(survey, scale_points = 5)
#'
#' # Summarize specific questions
#' summary_subset <- summarize_survey(survey, scale_points = 5,
#'   questions = c("My work is meaningful", "I feel valued"))
#'
#' # With favorability chart
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   summarize_survey(survey, scale_points = 5, plot = TRUE)
#' }
summarize_survey <- function(survey, scale_points, questions = "all",
                             emp_id_col = NULL, plot = FALSE) {
  if (plot) .check_ggplot2()
  if (!scale_points %in% 2:11) {
    stop("scale_points must be an integer between 2 and 11")
  }

  favorability <- get_favorability_map(scale_points)

  if (inherits(survey, "glint_survey")) {
    emp_id_col <- emp_id_col %||% survey$metadata$emp_id_col
    data <- survey$data
    all_questions <- survey$metadata$questions$question
  } else {
    data <- survey
    all_questions <- extract_questions(survey, emp_id_col)$question
  }

  if (length(questions) == 1 && questions == "all") {
    questions_to_analyze <- all_questions
  } else {
    missing_questions <- setdiff(questions, all_questions)
    if (length(missing_questions) > 0) {
      stop(sprintf(
        "Question(s) not found: %s\n\nAvailable questions:\n  %s",
        paste0("'", missing_questions, "'", collapse = ", "),
        paste0("  - ", all_questions, collapse = "\n")
      ))
    }
    questions_to_analyze <- questions
  }

  results <- purrr::map_dfr(questions_to_analyze, function(question_text) {
    responses <- data[[question_text]]
    n_total <- length(responses)

    valid_responses <- responses[!is.na(responses)]
    n_responses <- length(valid_responses)
    n_skips <- n_total - n_responses

    mean_response <- if (n_responses > 0) mean(valid_responses, na.rm = TRUE) else NA_real_
    sd_response <- if (n_responses > 1) sd(valid_responses, na.rm = TRUE) else NA_real_

    if (n_responses > 0) {
      n_favorable <- sum(valid_responses %in% favorability$favorable)
      n_neutral <- sum(valid_responses %in% favorability$neutral)
      n_unfavorable <- sum(valid_responses %in% favorability$unfavorable)

      pct_favorable <- round((n_favorable / n_responses) * 100, 1)
      pct_neutral <- round((n_neutral / n_responses) * 100, 1)
      pct_unfavorable <- round((n_unfavorable / n_responses) * 100, 1)
    } else {
      pct_favorable <- NA_real_
      pct_neutral <- NA_real_
      pct_unfavorable <- NA_real_
    }

    result <- dplyr::tibble(
      question = question_text,
      mean = mean_response,
      sd = sd_response,
      glint_score = mean_to_glint_score(mean_response, scale_points),
      n_responses = n_responses,
      n_skips = n_skips,
      n_total = n_total,
      pct_favorable = pct_favorable,
      pct_neutral = pct_neutral,
      pct_unfavorable = pct_unfavorable
    )

    return(result)
  })

  if (plot) {
    print(.plot_survey_summary(results))
    return(invisible(results))
  }

  return(results)
}


#' Get Response Distribution
#'
#' Calculates the distribution of response values for survey questions, returning
#' counts and percentages for each response value in tidy format.
#'
#' @param survey A glint_survey object or data frame containing survey data
#' @param questions Character vector of question text(s) to analyze, or "all" to
#'   analyze all questions (default: "all")
#' @param plot Logical. If \code{TRUE}, prints a stacked bar chart of response
#'   value distributions (red \eqn{\to} blue gradient) and returns the data
#'   invisibly. Requires \pkg{ggplot2}. Default: \code{FALSE}.
#'
#' @return A tibble with one row per question containing:
#'   \describe{
#'     \item{question}{The question text}
#'     \item{count_X}{Count of responses with value X (for each unique response value)}
#'     \item{pct_X}{Percentage of responses with value X (for each unique response value)}
#'   }
#'   When \code{plot = TRUE}, the same tibble is returned invisibly after
#'   printing the plot.
#'
#' @export
#'
#' @examples
#' survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
#' survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
#'
#' # Get response distribution for all questions
#' dist <- get_response_dist(survey)
#'
#' # Get response distribution for specific questions
#' dist_subset <- get_response_dist(survey,
#'   questions = c("My work is meaningful", "I feel valued"))
#'
#' # With distribution chart
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   get_response_dist(survey, plot = TRUE)
#' }
get_response_dist <- function(survey, questions = "all", plot = FALSE) {
  if (plot) .check_ggplot2()
  if (inherits(survey, "glint_survey")) {
    data <- survey$data
    all_questions <- survey$metadata$questions$question
  } else {
    data <- survey
    all_questions <- extract_questions(survey)$question
  }

  if (length(questions) == 1 && questions == "all") {
    questions_to_analyze <- all_questions
  } else {
    missing_questions <- setdiff(questions, all_questions)
    if (length(missing_questions) > 0) {
      stop(sprintf(
        "Question(s) not found: %s\n\nAvailable questions:\n  %s",
        paste0("'", missing_questions, "'", collapse = ", "),
        paste0("  - ", all_questions, collapse = "\n")
      ))
    }
    questions_to_analyze <- questions
  }

  results <- purrr::map_dfr(questions_to_analyze, function(question_text) {
    responses <- data[[question_text]]
    valid_responses <- responses[!is.na(responses)]

    if (length(valid_responses) > 0) {
      value_table <- table(valid_responses)
      value_counts <- as.list(value_table)
      names(value_counts) <- paste0("count_", names(value_counts))

      value_percents <- as.list(prop.table(value_table) * 100)
      names(value_percents) <- paste0("pct_", names(value_percents))
    } else {
      value_counts <- list()
      value_percents <- list()
    }

    result <- dplyr::tibble(question = question_text)
    result$value_counts <- list(value_counts)
    result$value_percents <- list(value_percents)

    return(result)
  })

  results <- expand_value_distributions(results)

  if (plot) {
    print(.plot_response_dist(results))
    return(invisible(results))
  }

  return(results)
}


#' Expand Value Distributions
#'
#' Internal function to expand value_counts and value_percents list columns
#' into separate columns for each response value.
#'
#' @param analysis_df A tibble from get_response_dist() with value_counts and value_percents list columns
#'
#' @return A tibble with expanded columns for each response value
#'
#' @keywords internal
expand_value_distributions <- function(analysis_df) {
  all_values <- unique(unlist(lapply(analysis_df$value_counts, function(vc) {
    as.numeric(stringr::str_remove(names(vc), "count_"))
  })))
  all_values <- sort(all_values)

  for (val in all_values) {
    count_name <- paste0("count_", val)
    pct_name <- paste0("pct_", val)

    analysis_df[[count_name]] <- sapply(analysis_df$value_counts, function(vc) {
      vc[[count_name]] %||% 0
    })

    analysis_df[[pct_name]] <- sapply(analysis_df$value_percents, function(vp) {
      vp[[pct_name]] %||% 0
    })
  }

  analysis_df$value_counts <- NULL
  analysis_df$value_percents <- NULL

  return(analysis_df)
}


#' Compare Survey Cycles
#'
#' Compares question-level metrics across multiple survey cycles, calculating
#' change scores and trends over time.
#'
#' @param ... Two or more glint_survey objects or data frames to compare
#' @param scale_points Integer specifying the number of scale points (2-11)
#' @param cycle_names Optional character vector of names for each cycle.
#'   If not provided, will use "Cycle 1", "Cycle 2", etc.
#' @param plot Logical. If \code{TRUE}, prints a line chart of Glint Score over
#'   cycles (one line per question) and returns the data invisibly. Requires
#'   \pkg{ggplot2}. Default: \code{FALSE}.
#'
#' @return A tibble with one row per question-cycle combination containing:
#'   \describe{
#'     \item{cycle}{The cycle name or number}
#'     \item{question}{The question text}
#'     \item{mean, sd, n_responses, n_skips, n_total}{Descriptive
#'       statistics for this question in this cycle (see \code{summarize_survey()})}
#'     \item{pct_favorable, pct_neutral, pct_unfavorable}{Favorability
#'       percentages for this question in this cycle}
#'     \item{change_from_previous}{Change in raw mean score from the previous
#'       cycle (NA for the first cycle)}
#'     \item{pct_change_from_previous}{Percentage change in raw mean score from
#'       the previous cycle (NA for the first cycle)}
#'     \item{glint_score_change_from_previous}{Change in Glint Score (0-100
#'       scale) from the previous cycle (NA for the first cycle)}
#'   }
#'   When \code{plot = TRUE}, the same tibble is returned invisibly after
#'   printing the plot.
#'
#' @export
#'
#' @examples
#' survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
#' survey1 <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
#' survey2 <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
#' survey3 <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
#'
#' comparison <- compare_cycles(survey1, survey2, survey3,
#'                               scale_points = 5,
#'                               cycle_names = c("Q1 2024", "Q2 2024", "Q3 2024"))
#' print(comparison)
#'
#' # With trend chart
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   compare_cycles(survey1, survey2, survey3, scale_points = 5,
#'                  cycle_names = c("Q1 2024", "Q2 2024", "Q3 2024"),
#'                  plot = TRUE)
#' }
compare_cycles <- function(..., scale_points, cycle_names = NULL,
                           plot = FALSE) {
  if (plot) .check_ggplot2()
  surveys <- list(...)

  if (length(surveys) < 2) {
    stop("At least two surveys are required for comparison")
  }

  if (is.null(cycle_names)) {
    cycle_names <- paste("Cycle", seq_along(surveys))
  } else if (length(cycle_names) != length(surveys)) {
    stop("Number of cycle_names must match number of surveys")
  }

  analyses <- purrr::map2_dfr(surveys, cycle_names, function(survey, cycle_name) {
    result <- summarize_survey(survey, scale_points = scale_points)
    result$cycle <- cycle_name
    result
  })

  # Reorder columns to put cycle first
  analyses <- analyses %>%
    dplyr::select(cycle, question, dplyr::everything())

  analyses <- analyses %>%
    dplyr::arrange(question, cycle) %>%
    dplyr::group_by(question) %>%
    dplyr::mutate(
      change_from_previous = mean - dplyr::lag(mean),
      pct_change_from_previous = (mean - dplyr::lag(mean)) / dplyr::lag(mean) * 100,
      glint_score_change_from_previous = glint_score - dplyr::lag(glint_score)
    ) %>%
    dplyr::ungroup()

  if (plot) {
    print(.plot_compare_cycles(analyses, cycle_names))
    return(invisible(analyses))
  }

  return(analyses)
}


#' Calculate Question Correlations
#'
#' Calculates correlations between all question response columns in the survey.
#' Supports multiple correlation methods and output formats.
#'
#' @param survey A glint_survey object or data frame containing survey data
#' @param method Character string indicating the correlation method: "pearson"
#'   (default), "spearman", or "kendall"
#' @param format Character string indicating output format: "long" for long format
#'   with one row per question pair (default), or "matrix" for traditional
#'   correlation matrix
#' @param use Character string indicating how to handle missing values, passed to
#'   cor() function (default: "pairwise.complete.obs")
#' @param plot Logical. If \code{TRUE}, prints a correlation heatmap and returns
#'   the data invisibly. Only supported when \code{format = "long"} (the
#'   default); ignored with a warning when \code{format = "matrix"}. Requires
#'   \pkg{ggplot2}. Default: \code{FALSE}.
#'
#' @return If format = "long", a tibble with columns:
#'   \describe{
#'     \item{question1}{First question text}
#'     \item{question2}{Second question text}
#'     \item{correlation}{Correlation coefficient}
#'     \item{p_value}{P-value for test of correlation significance}
#'     \item{n}{Number of complete pairs used in calculation}
#'   }
#'   If format = "matrix", a matrix with questions as rows and columns.
#'   When \code{plot = TRUE} and \code{format = "long"}, the tibble is returned
#'   invisibly after printing the plot.
#'
#' @export
#'
#' @examples
#' survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
#' survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
#'
#' # Get Pearson correlations in long format (default)
#' correlations <- get_correlations(survey)
#'
#' # Get Spearman correlations
#' correlations_spearman <- get_correlations(survey, method = "spearman")
#'
#' # Get correlation matrix
#' cor_matrix <- get_correlations(survey, format = "matrix")
#'
#' # With correlation heatmap
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   get_correlations(survey, plot = TRUE)
#' }
get_correlations <- function(survey, method = "pearson", format = "long",
                             use = "pairwise.complete.obs", plot = FALSE) {
  if (plot) .check_ggplot2()
  if (!method %in% c("spearman", "pearson", "kendall")) {
    stop("method must be one of: 'spearman', 'pearson', or 'kendall'")
  }

  if (!format %in% c("long", "matrix")) {
    stop("format must be one of: 'long' or 'matrix'")
  }

  if (inherits(survey, "glint_survey")) {
    data <- survey$data
    questions <- survey$metadata$questions$question
  } else {
    data <- survey
    questions <- extract_questions(data)$question
  }

  response_data <- data[, questions, drop = FALSE]
  cor_matrix <- cor(response_data, method = method, use = use)

  if (format == "matrix") {
    if (plot) {
      warning("plot = TRUE is not supported when format = 'matrix'. Returning matrix without plot.")
    }
    return(cor_matrix)
  } else {
    long_data <- expand.grid(
      question1 = questions,
      question2 = questions,
      stringsAsFactors = FALSE
    )

    long_data$correlation <- as.vector(cor_matrix)
    long_data$p_value <- NA_real_
    long_data$n <- NA_integer_

    for (i in seq_len(nrow(long_data))) {
      q1 <- long_data$question1[i]
      q2 <- long_data$question2[i]

      x <- response_data[[q1]]
      y <- response_data[[q2]]
      complete_pairs <- !is.na(x) & !is.na(y)
      n_complete <- sum(complete_pairs)

      long_data$n[i] <- n_complete

      if (n_complete > 2 && q1 != q2) {
        test_result <- cor.test(
          x[complete_pairs],
          y[complete_pairs],
          method = method,
          exact = FALSE  # Use asymptotic p-value for consistency
        )
        long_data$p_value[i] <- test_result$p.value
      } else if (q1 == q2) {
        # Self-correlation always has p-value of 0
        long_data$p_value[i] <- 0
      }
    }

    long_data <- dplyr::as_tibble(long_data) %>%
      dplyr::select(question1, question2, correlation, p_value, n)

    if (plot) {
      print(.plot_correlations(long_data))
      return(invisible(long_data))
    }

    return(long_data)
  }
}


#' Extract Survey Factors
#'
#' Performs factor analysis on survey question responses to identify underlying
#' latent factors. Supports multiple rotation methods with oblique rotation as default.
#'
#' @param survey A glint_survey object or data frame containing survey data
#' @param n_factors Integer indicating the number of factors to extract. If NULL
#'   (default), will use parallel analysis to determine optimal number of factors
#' @param rotation Character string indicating rotation method: "oblimin" (default),
#'   "varimax", "promax", "quartimax", "equamax", or "none"
#' @param min_loading Minimum factor loading to display in results (default: 0.3)
#' @param fm Character string indicating factoring method: "minres" (minimum
#'   residuals, default), "ml" (maximum likelihood), "pa" (principal axis),
#'   "wls" (weighted least squares), "gls" (generalized least squares), or
#'   "uls" (unweighted least squares)
#' @param plot Logical. If \code{TRUE}, prints a factor loading heatmap and
#'   returns the result list invisibly. Requires \pkg{ggplot2}. Default:
#'   \code{FALSE}.
#'
#' @return A list containing:
#'   \describe{
#'     \item{factor_summary}{A tibble with one row per question-factor combination
#'       (filtered to \code{abs(loading) >= min_loading}), containing:
#'       \code{question} (item text), \code{factor} (factor name),
#'       \code{loading} (loading coefficient), \code{loading_label}
#'       ("Strong" >= 0.75 / "Medium" 0.60-0.74 / "Weak" < 0.60),
#'       \code{communality} (proportion of item variance explained by all factors),
#'       and \code{factor_variance_pct} (percentage of total variance explained
#'       by this factor). Sorted by factor then descending loading strength.}
#'     \item{fa_object}{Original fa object from psych package for further analysis}
#'   }
#'   When \code{plot = TRUE}, the same list is returned invisibly after printing
#'   the plot.
#'
#' @export
#'
#' @examples
#' survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
#' survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
#'
#' if (requireNamespace("psych", quietly = TRUE)) {
#'   # Extract factors with fixed count to keep runtime small
#'   factors <- extract_survey_factors(survey, n_factors = 1)
#'   print(factors$factor_summary)
#'
#'   # Filter to strong loaders only
#'   strong <- dplyr::filter(factors$factor_summary, loading_label == "Strong")
#'
#'   # With factor loading heatmap
#'   if (requireNamespace("ggplot2", quietly = TRUE)) {
#'     extract_survey_factors(survey, n_factors = 1, plot = TRUE)
#'   }
#' }
extract_survey_factors <- function(survey, n_factors = NULL, rotation = "oblimin",
                                   min_loading = 0.3, fm = "minres",
                                   plot = FALSE) {
  if (plot) .check_ggplot2()
  if (!requireNamespace("psych", quietly = TRUE)) {
    stop("Package 'psych' is required for factor analysis. Install it with: install.packages('psych')")
  }

  valid_rotations <- c("oblimin", "varimax", "promax", "quartimax", "equamax", "none")
  if (!rotation %in% valid_rotations) {
    stop(sprintf(
      "rotation must be one of: %s",
      paste0("'", valid_rotations, "'", collapse = ", ")
    ))
  }

  valid_fm <- c("minres", "ml", "pa", "wls", "gls", "uls")
  if (!fm %in% valid_fm) {
    stop(sprintf(
      "fm must be one of: %s",
      paste0("'", valid_fm, "'", collapse = ", ")
    ))
  }

  if (inherits(survey, "glint_survey")) {
    data <- survey$data
    questions <- survey$metadata$questions$question
  } else {
    data <- survey
    questions <- extract_questions(data)$question
  }

  response_data <- data[, questions, drop = FALSE]

  # Remove cases with all missing values
  response_data <- response_data[rowSums(!is.na(response_data)) > 0, ]

  if (is.null(n_factors)) {
    message("Determining optimal number of factors using parallel analysis...")
    pa_result <- psych::fa.parallel(
      response_data,
      fa = "fa",
      fm = fm,
      plot = FALSE,
      n.iter = 20
    )
    n_factors <- pa_result$nfact
    message(sprintf("Parallel analysis suggests %d factor(s)", n_factors))
  }

  if (n_factors < 1 || n_factors > ncol(response_data)) {
    stop(sprintf(
      "n_factors must be between 1 and %d (number of questions)",
      ncol(response_data)
    ))
  }

  fa_result <- psych::fa(
    response_data,
    nfactors = n_factors,
    rotate = rotation,
    fm = fm
  )

  loadings_matrix <- fa_result$loadings
  class(loadings_matrix) <- "matrix"

  # Build long-format loadings (all question × factor combinations)
  loadings_long <- expand.grid(
    question = rownames(loadings_matrix),
    factor = colnames(loadings_matrix),
    stringsAsFactors = FALSE
  )
  loadings_long$loading <- as.vector(loadings_matrix)

  # Build communality lookup (per question)
  communality_lookup <- dplyr::tibble(
    question = names(fa_result$communality),
    communality = fa_result$communality
  )

  # Build factor variance lookup (per factor)
  if (n_factors == 1) {
    factor_variance_pct <- fa_result$values[1] / ncol(response_data) * 100
  } else {
    factor_variance_pct <- fa_result$Vaccount["Proportion Var", ] * 100
  }
  variance_lookup <- dplyr::tibble(
    factor = colnames(loadings_matrix),
    factor_variance_pct = factor_variance_pct
  )

  # Assemble consolidated factor summary
  factor_summary <- dplyr::as_tibble(loadings_long) %>%
    dplyr::filter(abs(loading) >= min_loading) %>%
    dplyr::mutate(
      loading_label = dplyr::case_when(
        abs(loading) >= 0.75 ~ "Strong",
        abs(loading) <  0.60 ~ "Weak",
        TRUE                 ~ "Medium"
      )
    ) %>%
    dplyr::left_join(communality_lookup, by = "question") %>%
    dplyr::left_join(variance_lookup, by = "factor") %>%
    dplyr::arrange(factor, dplyr::desc(abs(loading))) %>%
    dplyr::select(question, factor, loading, loading_label, communality,
                  factor_variance_pct)

  result <- list(
    factor_summary = factor_summary,
    fa_object = fa_result
  )

  class(result) <- c("survey_factors", "list")

  if (plot) {
    print(.plot_survey_factors(result$factor_summary))
    return(invisible(result))
  }

  return(result)
}


#' Analyze Employee Attrition
#'
#' Analyzes the relationship between survey responses and employee attrition,
#' calculating how much more (or less) likely employees are to leave within
#' specified time periods if they respond unfavorably vs favorably to survey questions.
#'
#' @param survey A glint_survey object or data frame containing survey data
#' @param attrition_file Path to CSV file containing employee attrition data
#' @param emp_id_col Character string specifying the column name for employee ID
#' @param term_date_col Character string specifying the column name for termination date
#' @param scale_points Integer specifying the number of scale points (2-11)
#' @param time_periods Integer vector specifying time periods in days to analyze
#'   (default: c(90, 180, 365))
#' @param attribute_cols Optional character vector of attribute column names to
#'   segment results by (e.g., \code{c("Department", "Gender")}). Attributes must
#'   already be joined to the survey via \code{join_attributes()}. When \code{NULL}
#'   (default), results are returned for the overall population.
#' @param min_group_size Minimum number of employees required in an attribute group
#'   for it to be included in results (default: 5). Ignored when
#'   \code{attribute_cols = NULL}.
#' @param plot Logical. If \code{TRUE}, prints a grouped bar chart of favorable
#'   vs. unfavorable attrition rates faceted by time period and returns the data
#'   invisibly. When \code{attribute_cols} is supplied, the first attribute
#'   column is used as an additional facet dimension. Requires \pkg{ggplot2}.
#'   Default: \code{FALSE}.
#'
#' @return A tibble with one row per (attribute group)-question-time period
#'   combination containing:
#'   \describe{
#'     \item{attribute columns}{One column per entry in \code{attribute_cols}
#'       (only present when \code{attribute_cols} is supplied)}
#'     \item{group_size}{Number of employees in the attribute group (only present
#'       when \code{attribute_cols} is supplied)}
#'     \item{question}{The question text}
#'     \item{days}{The time period in days}
#'     \item{favorable_n}{Number of employees who responded favorably}
#'     \item{favorable_attrition}{Proportion who left within time period (favorable)}
#'     \item{unfavorable_n}{Number of employees who responded unfavorably}
#'     \item{unfavorable_attrition}{Proportion who left within time period (unfavorable)}
#'     \item{attrition_ratio}{Ratio of unfavorable to favorable attrition rates}
#'   }
#'   When \code{plot = TRUE}, the same tibble is returned invisibly after
#'   printing the plot.
#'
#' @export
#'
#' @examples
#' survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
#' attrition_file <- system.file("extdata", "attrition.csv", package = "vivaglint")
#' attr_path <- system.file("extdata", "employee_attributes.csv", package = "vivaglint")
#' survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
#'
#' # Overall attrition analysis
#' attrition <- analyze_attrition(
#'   survey,
#'   attrition_file = attrition_file,
#'   emp_id_col = "EMP ID",
#'   term_date_col = "Termination Date",
#'   scale_points = 5
#' )
#'
#' # Attrition segmented by Department and Gender
#' survey_enriched <- join_attributes(survey, attr_path, emp_id_col = "EMP ID")
#' attrition_by_dept <- analyze_attrition(
#'   survey_enriched,
#'   attrition_file = attrition_file,
#'   emp_id_col = "EMP ID",
#'   term_date_col = "Termination Date",
#'   scale_points = 5,
#'   attribute_cols = c("Department", "Gender"),
#'   min_group_size = 2
#' )
#'
#' # With attrition chart
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   analyze_attrition(survey, attrition_file = attrition_file,
#'                     emp_id_col = "EMP ID", term_date_col = "Termination Date",
#'                     scale_points = 5, plot = TRUE)
#' }
analyze_attrition <- function(survey, attrition_file, emp_id_col = NULL, term_date_col,
                              scale_points, time_periods = c(90, 180, 365),
                              attribute_cols = NULL, min_group_size = 5,
                              plot = FALSE) {
  if (plot) .check_ggplot2()
  if (!scale_points %in% 2:11) {
    stop("scale_points must be an integer between 2 and 11")
  }

  favorability <- get_favorability_map(scale_points)

  if (inherits(survey, "glint_survey")) {
    emp_id_col <- emp_id_col %||% survey$metadata$emp_id_col
    survey_data <- survey$data
    questions <- survey$metadata$questions$question
    col_map <- survey$metadata$standard_column_map %||% list()
    completion_date_col <- col_map$completion_date %||% "Survey Cycle Completion Date"
  } else {
    survey_data <- survey
    questions <- extract_questions(survey)$question
    completion_date_col <- "Survey Cycle Completion Date"
  }

  if (is.null(emp_id_col)) {
    stop("emp_id_col must be specified when survey is a plain data frame")
  }
  if (is.null(completion_date_col) || !nzchar(completion_date_col)) {
    stop(
      "analyze_attrition() requires a survey completion date column. ",
      "Provide completion_date_col when reading the survey.",
      call. = FALSE
    )
  }

  if (!file.exists(attrition_file)) {
    stop(sprintf("Attrition file not found: '%s'", attrition_file))
  }

  attrition_data <- readr::read_csv(attrition_file, show_col_types = FALSE)

  if (!emp_id_col %in% names(attrition_data)) {
    stop(sprintf("Column '%s' not found in attrition file", emp_id_col))
  }
  if (!term_date_col %in% names(attrition_data)) {
    stop(sprintf("Column '%s' not found in attrition file", term_date_col))
  }
  if (!emp_id_col %in% names(survey_data)) {
    stop(sprintf("Column '%s' not found in survey data", emp_id_col))
  }

  # Parse termination dates (try multiple formats)
  attrition_data[[term_date_col]] <- tryCatch(
    {
      # Try different date parsing functions
      parsed <- lubridate::parse_date_time(
        attrition_data[[term_date_col]],
        orders = c("ymd", "mdy", "dmy", "ymd HMS", "mdy HMS", "dmy HMS")
      )
      as.Date(parsed)
    },
    error = function(e) {
      stop(sprintf(
        "Error parsing termination dates in column '%s': %s",
        term_date_col, e$message
      ))
    }
  )

  if (!completion_date_col %in% names(survey_data)) {
    stop(
      "Survey data must contain the completion date column: '",
      completion_date_col,
      "'.\n\nColumns detected in the data frame:\n",
      paste0("  - ", names(survey_data), collapse = "\n"),
      "\n\nTo resolve, pass completion_date_col when reading with read_glint_survey() or read_glint_survey_api(), or rename your columns.",
      call. = FALSE
    )
  }

  combined_data <- survey_data %>%
    dplyr::left_join(
      attrition_data %>% dplyr::select(dplyr::all_of(c(emp_id_col, term_date_col))),
      by = emp_id_col
    )

  combined_data$survey_date <- as.Date(combined_data[[completion_date_col]])
  combined_data$days_to_term <- as.numeric(
    combined_data[[term_date_col]] - combined_data$survey_date
  )

  if (!is.null(attribute_cols)) {
    missing_attr <- setdiff(attribute_cols, names(combined_data))
    if (length(missing_attr) > 0) {
      stop(paste0(
        "Attribute column(s) not found in survey data: ",
        paste(missing_attr, collapse = ", "),
        "\nDid you call join_attributes() first?"
      ))
    }
  }

  # Core analysis: one row per question × time period for a given data slice
  run_attrition_core <- function(data_slice) {
    purrr::map_dfr(questions, function(question_text) {
      responses <- data_slice[[question_text]]

      response_class <- dplyr::case_when(
        responses %in% favorability$favorable   ~ "favorable",
        responses %in% favorability$unfavorable ~ "unfavorable",
        TRUE                                    ~ "neutral"
      )

      purrr::map_dfr(time_periods, function(days) {
        left_within_period <- !is.na(data_slice$days_to_term) &
          data_slice$days_to_term > 0 &
          data_slice$days_to_term <= days

        favorable_mask <- response_class == "favorable" & !is.na(responses)
        favorable_n <- sum(favorable_mask)
        favorable_attrition <- if (favorable_n > 0) {
          sum(left_within_period[favorable_mask]) / favorable_n
        } else {
          NA_real_
        }

        unfavorable_mask <- response_class == "unfavorable" & !is.na(responses)
        unfavorable_n <- sum(unfavorable_mask)
        unfavorable_attrition <- if (unfavorable_n > 0) {
          sum(left_within_period[unfavorable_mask]) / unfavorable_n
        } else {
          NA_real_
        }

        attrition_ratio <- if (!is.na(favorable_attrition) && !is.na(unfavorable_attrition)) {
          if (favorable_attrition == 0 && unfavorable_attrition == 0) {
            NA_real_
          } else if (favorable_attrition == 0 && unfavorable_attrition > 0) {
            Inf
          } else {
            unfavorable_attrition / favorable_attrition
          }
        } else {
          NA_real_
        }

        dplyr::tibble(
          question             = question_text,
          days                 = days,
          favorable_n          = favorable_n,
          favorable_attrition  = round(favorable_attrition, 4),
          unfavorable_n        = unfavorable_n,
          unfavorable_attrition = round(unfavorable_attrition, 4),
          attrition_ratio      = ifelse(is.finite(attrition_ratio),
                                        round(attrition_ratio, 2),
                                        attrition_ratio)
        )
      })
    })
  }

  # Overall analysis (no attributes)
  if (is.null(attribute_cols)) {
    results <- run_attrition_core(combined_data)
    if (plot) {
      print(.plot_attrition(results, NULL))
      return(invisible(results))
    }
    return(results)
  }

  # Attribute-segmented analysis
  attribute_groups <- combined_data %>%
    dplyr::group_by(dplyr::across(dplyr::all_of(attribute_cols))) %>%
    dplyr::summarise(group_size = dplyr::n(), .groups = "drop") %>%
    dplyr::filter(group_size >= min_group_size)

  if (nrow(attribute_groups) == 0) {
    warning("No attribute groups meet the minimum size threshold")
    return(dplyr::tibble())
  }

  results <- purrr::map_dfr(seq_len(nrow(attribute_groups)), function(i) {
    group_values <- attribute_groups[i, attribute_cols, drop = FALSE]
    group_size   <- attribute_groups$group_size[i]

    group_data <- combined_data
    for (col in attribute_cols) {
      group_data <- group_data %>%
        dplyr::filter(!!rlang::sym(col) == group_values[[col]])
    }

    dplyr::bind_cols(
      group_values,
      dplyr::tibble(group_size = group_size),
      run_attrition_core(group_data)
    )
  })

  if (plot) {
    print(.plot_attrition(results, attribute_cols))
    return(invisible(results))
  }

  results
}


#' Analyze Survey Responses by Attributes
#'
#' Aggregates survey responses by employee attributes (e.g., Department,
#' Gender, Tenure) and calculates the same metrics as \code{summarize_survey()}
#' for each attribute group combination. Only groups meeting the minimum size
#' threshold are included in the results.
#'
#' Attributes can be supplied in two ways:
#' \enumerate{
#'   \item Pass \code{attribute_file} (a file path or data frame) and the join
#'     is performed internally on each call.
#'   \item Pre-join attributes once with \code{join_attributes()} and pass the
#'     enriched survey directly — omit \code{attribute_file} entirely. This is
#'     more efficient when calling \code{analyze_by_attributes()} multiple times
#'     or when you want to filter the data before analysis.
#' }
#'
#' @param survey A \code{glint_survey} object or data frame. If attributes have
#'   already been joined via \code{join_attributes()}, \code{attribute_file} can
#'   be omitted.
#' @param attribute_file Optional. A file path (character string) or data frame
#'   containing employee attributes to join. If \code{NULL} (default), the
#'   survey must already contain the columns named in \code{attribute_cols}.
#' @param scale_points Integer specifying the number of scale points (2-11)
#' @param attribute_cols Character vector of column names to group by
#'   (e.g., \code{c("Department", "Gender", "Tenure Group")})
#' @param emp_id_col Character string specifying the employee ID column name
#'   in both the survey data and the attribute data
#' @param min_group_size Integer specifying the minimum number of employees
#'   required for a group to be included in results (default: 5)
#' @param plot Logical. If \code{TRUE}, prints a faceted dot plot of Glint
#'   Scores by attribute group (one facet per question) and returns the data
#'   invisibly. When multiple \code{attribute_cols} are supplied, only the first
#'   is plotted. Requires \pkg{ggplot2}. Default: \code{FALSE}.
#'
#' @return A tibble with one row per attribute-group-question combination
#'   containing:
#'   \describe{
#'     \item{attribute columns}{Values for each attribute grouping variable}
#'     \item{group_size}{Number of employees in this attribute group}
#'     \item{question, mean, sd, n_responses, n_skips, n_total}{
#'       Descriptive statistics for this group on this question}
#'     \item{pct_favorable, pct_neutral, pct_unfavorable}{Favorability
#'       percentages for this group on this question}
#'   }
#'   When \code{plot = TRUE}, the same tibble is returned invisibly after
#'   printing the plot.
#'
#' @export
#'
#' @examples
#' survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
#' attr_file <- system.file("extdata", "employee_attributes.csv", package = "vivaglint")
#' survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
#'
#' # Option 1: provide attribute_file directly
#' results <- analyze_by_attributes(
#'   survey,
#'   attribute_file = attr_file,
#'   scale_points = 5,
#'   attribute_cols = "Department",
#'   emp_id_col = "EMP ID",
#'   min_group_size = 2
#' )
#'
#' # Option 2: pre-join with join_attributes(), then omit attribute_file
#' survey_enriched <- join_attributes(survey, attr_file, emp_id_col = "EMP ID")
#' results <- analyze_by_attributes(survey_enriched, scale_points = 5,
#'                                  attribute_cols = "Department",
#'                                  emp_id_col = "EMP ID",
#'                                  min_group_size = 2)
#'
#' # With dot plot
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   analyze_by_attributes(survey_enriched, scale_points = 5,
#'                         attribute_cols = "Department",
#'                         emp_id_col = "EMP ID", plot = TRUE,
#'                         min_group_size = 2)
#' }
analyze_by_attributes <- function(survey, attribute_file = NULL, scale_points,
                                 attribute_cols, emp_id_col = NULL,
                                 min_group_size = 5, plot = FALSE) {
  if (plot) .check_ggplot2()
  if (!scale_points %in% 2:11) {
    stop("scale_points must be an integer between 2 and 11")
  }

  if (inherits(survey, "glint_survey")) {
    emp_id_col <- emp_id_col %||% survey$metadata$emp_id_col
  }

  if (is.null(emp_id_col)) {
    stop("emp_id_col must be specified when survey is a plain data frame")
  }

  if (!is.null(attribute_file)) {
    survey <- join_attributes(survey, attribute_file, emp_id_col = emp_id_col)
  }

  if (inherits(survey, "glint_survey")) {
    survey_data <- survey$data
  } else {
    survey_data <- survey
  }

  missing_cols <- setdiff(attribute_cols, names(survey_data))
  if (length(missing_cols) > 0) {
    stop(paste0(
      "Attribute column(s) not found in survey data: ",
      paste(missing_cols, collapse = ", "),
      "\nDid you forget to supply attribute_file or call join_attributes() first?"
    ))
  }

  # Use survey_data directly as combined_data — attributes already joined
  combined_data <- survey_data

  attribute_groups <- combined_data %>%
    dplyr::select(dplyr::all_of(c(attribute_cols, emp_id_col))) %>%
    dplyr::distinct() %>%
    dplyr::group_by(dplyr::across(dplyr::all_of(attribute_cols))) %>%
    dplyr::summarise(
      group_size = dplyr::n(),
      .groups = "drop"
    ) %>%
    dplyr::filter(group_size >= min_group_size)

  if (nrow(attribute_groups) == 0) {
    warning("No attribute groups meet the minimum size threshold")
    return(dplyr::tibble())
  }

  # Get question columns (excluding standard columns AND ALL joined attribute columns,
  # not just the ones used for grouping in this call)
  standard_cols <- if (inherits(survey, "glint_survey")) {
    survey$metadata$standard_columns %||% get_standard_columns(emp_id_col)
  } else {
    get_standard_columns(emp_id_col)
  }
  all_cols <- names(survey_data)
  all_attr_cols <- if (inherits(survey, "glint_survey")) {
    survey$metadata$attribute_cols %||% attribute_cols
  } else {
    attribute_cols
  }
  question_cols <- setdiff(all_cols, c(standard_cols, all_attr_cols))

  results <- purrr::map_dfr(seq_len(nrow(attribute_groups)), function(i) {
    group_values <- attribute_groups[i, attribute_cols, drop = FALSE]
    group_size <- attribute_groups$group_size[i]

    group_data <- combined_data
    for (col in attribute_cols) {
      group_data <- group_data %>%
        dplyr::filter(!!rlang::sym(col) == group_values[[col]])
    }

    group_data_survey <- group_data %>%
      dplyr::select(dplyr::all_of(c(standard_cols, question_cols)))

    group_summary <- summarize_survey(group_data_survey, scale_points = scale_points, questions = "all", emp_id_col = emp_id_col)

    group_summary <- dplyr::bind_cols(
      group_values,
      dplyr::tibble(group_size = group_size),
      group_summary
    )

    return(group_summary)
  })

  if (plot) {
    print(.plot_by_attributes(results, attribute_cols))
    return(invisible(results))
  }

  return(results)
}


#' Search Survey Comments
#'
#' Searches through all survey comment text and returns matching responses
#' with their associated question text, numeric response values, comments, and
#' topics. Supports both exact substring matching and fuzzy (approximate)
#' matching that tolerates minor spelling differences.
#'
#' @param survey A glint_survey object or data frame containing survey data
#' @param query Character string to search for within comments
#' @param exact Logical. If \code{TRUE}, performs case-sensitive exact substring
#'   matching. If \code{FALSE} (default), performs case-insensitive approximate
#'   matching that also tolerates minor spelling differences.
#' @param max_distance Numeric between 0 and 1 controlling fuzzy match tolerance
#'   when \code{exact = FALSE}. Higher values allow more differences (e.g. more
#'   typos). Default is 0.2.
#'
#' @return A tibble with one row per matching comment containing:
#'   \describe{
#'     \item{question}{The survey question text}
#'     \item{response}{The numeric survey response value}
#'     \item{comment}{The comment text that matched the query}
#'     \item{topics}{The comment topic(s) associated with the comment}
#'   }
#'   Returns an empty tibble with the same columns if no matches are found.
#'
#' @export
#'
#' @examples
#' survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
#' survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
#'
#' # Fuzzy search (default) - finds "manager", "Manager", "managers", etc.
#' results <- search_comments(survey, "manager")
#'
#' # Exact search - finds only the literal string "Manager" (case-sensitive)
#' results_exact <- search_comments(survey, "Manager", exact = TRUE)
#'
#' # Wider fuzzy tolerance to catch more spelling variations
#' results_wide <- search_comments(survey, "management", max_distance = 0.3)
search_comments <- function(survey, query, exact = FALSE, max_distance = 0.2) {
  if (!is.character(query) || length(query) != 1 || nchar(trimws(query)) == 0) {
    stop("query must be a non-empty character string")
  }
  if (!is.logical(exact) || length(exact) != 1) {
    stop("exact must be TRUE or FALSE")
  }
  if (!is.numeric(max_distance) || length(max_distance) != 1 ||
      max_distance < 0 || max_distance > 1) {
    stop("max_distance must be a number between 0 and 1")
  }

  if (inherits(survey, "glint_survey")) {
    data <- survey$data
    all_questions <- survey$metadata$questions$question
  } else {
    data <- survey
    all_questions <- extract_questions(survey)$question
  }

  results <- purrr::map_dfr(all_questions, function(question_text) {
    comment_col <- paste0(question_text, "_COMMENT")
    topics_col  <- paste0(question_text, "_COMMENT_TOPICS")

    if (!comment_col %in% names(data)) return(NULL)

    comments  <- as.character(data[[comment_col]])
    responses <- data[[question_text]]
    topics    <- if (topics_col %in% names(data)) {
      as.character(data[[topics_col]])
    } else {
      rep(NA_character_, nrow(data))
    }

    # Only consider rows with non-empty comments
    has_comment <- !is.na(comments) & nchar(trimws(comments)) > 0
    if (!any(has_comment)) return(NULL)

    if (exact) {
      # Case-sensitive literal substring match
      matched <- has_comment & grepl(query, comments, fixed = TRUE)
    } else {
      # Case-insensitive partial match
      partial_match <- has_comment & grepl(query, comments, ignore.case = TRUE)

      # Approximate match via agrep on non-empty comments only
      fuzzy_match <- logical(length(comments))
      candidate_idx <- which(has_comment)
      if (length(candidate_idx) > 0) {
        hits <- agrep(query, comments[candidate_idx],
                      max.distance = max_distance, ignore.case = TRUE)
        if (length(hits) > 0) {
          fuzzy_match[candidate_idx[hits]] <- TRUE
        }
      }

      matched <- partial_match | fuzzy_match
    }

    if (!any(matched)) return(NULL)

    dplyr::tibble(
      question = question_text,
      response = responses[matched],
      comment  = comments[matched],
      topics   = topics[matched]
    )
  })

  if (is.null(results) || nrow(results) == 0) {
    return(dplyr::tibble(
      question = character(),
      response = numeric(),
      comment  = character(),
      topics   = character()
    ))
  }

  return(results)
}


#' Split Survey Data into Quantitative and Qualitative Components
#'
#' Separates a survey dataset into two data frames: one containing only numeric
#' response data (for statistical analysis) and one containing only comment
#' and topic data (for qualitative analysis). Both outputs retain the employee
#' ID column so they can be rejoined at any time.
#'
#' @param survey A glint_survey object or data frame containing survey data
#' @param emp_id_col Character string specifying the employee ID column name
#'
#' @return A named list with two elements:
#'   \describe{
#'     \item{quantitative}{A tibble with all standard respondent columns plus
#'       one numeric response column per question. Comment, topic, and sensitive
#'       flag columns are excluded.}
#'     \item{qualitative}{A tibble with the employee ID column plus the comment,
#'       topic, and sensitive flag columns for every question. Numeric response
#'       columns are excluded.}
#'   }
#'
#' @export
#'
#' @examples
#' survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
#' survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
#' parts  <- split_survey_data(survey)
#'
#' # Analyze numeric responses
#' summary <- summarize_survey(parts$quantitative, scale_points = 5,
#'                             emp_id_col = "EMP ID")
#'
#' # Work with comments separately
#' comments <- parts$qualitative
split_survey_data <- function(survey, emp_id_col = NULL) {
  if (inherits(survey, "glint_survey")) {
    emp_id_col <- emp_id_col %||% survey$metadata$emp_id_col
    data <- survey$data
  } else {
    data <- survey
  }

  if (is.null(emp_id_col)) {
    stop("emp_id_col could not be determined. Load your survey with read_glint_survey() and specify emp_id_col.")
  }

  standard_cols <- if (inherits(survey, "glint_survey")) {
    survey$metadata$standard_columns %||% get_standard_columns(emp_id_col)
  } else {
    get_standard_columns(emp_id_col)
  }
  questions     <- extract_questions(data, emp_id_col)

  response_cols <- questions$response_col
  comment_cols  <- questions$comment_col[questions$comment_col %in% names(data)]
  topics_cols   <- questions$topics_col[questions$topics_col   %in% names(data)]
  flag_cols     <- questions$flag_col[questions$flag_col       %in% names(data)]

  # Quantitative: standard columns + one numeric response column per question
  quant_cols <- c(
    standard_cols[standard_cols %in% names(data)],
    response_cols[response_cols %in% names(data)]
  )

  # Qualitative: employee ID (for joining) + all comment/topic/flag columns
  qual_cols <- c(emp_id_col, comment_cols, topics_cols, flag_cols)

  list(
    quantitative = dplyr::select(data, dplyr::all_of(quant_cols)),
    qualitative  = dplyr::select(data, dplyr::all_of(qual_cols))
  )
}


`%||%` <- function(x, y) {
  if (is.null(x)) y else x
}

#' Configure Viva Glint API credentials
#'
#' Stores Viva Glint API credentials in environment variables for the current
#' R session. Optionally writes the values to ~/.Renviron for persistence.
#'
#' Required environment variables:
#' - GLINT_TENANT_ID
#' - GLINT_CLIENT_ID
#' - GLINT_CLIENT_SECRET
#' - GLINT_EXPERIENCE_NAME
#'
#' @param tenant_id Azure AD tenant ID
#' @param client_id Azure AD app (client) ID
#' @param client_secret Azure AD app client secret
#' @param experience_name Viva Glint experience name (e.g., "contoso@demo")
#' @param save_to_renviron Logical; if TRUE, append values to ~/.Renviron
#'
#' @return Invisibly returns TRUE after saving credentials
#'
#' @export
#'
#' @examples
#' \dontrun{
#' glint_setup(
#'   tenant_id = "your-tenant-id",
#'   client_id = "your-client-id",
#'   client_secret = "your-client-secret",
#'   experience_name = "your-experience-name",
#'   save_to_renviron = TRUE
#' )
#' }
glint_setup <- function(tenant_id,
                        client_id,
                        client_secret,
                        experience_name,
                        save_to_renviron = FALSE) {
  if (!nzchar(tenant_id)) {
    stop("tenant_id must be provided.", call. = FALSE)
  }
  if (!nzchar(client_id)) {
    stop("client_id must be provided.", call. = FALSE)
  }
  if (!nzchar(client_secret)) {
    stop("client_secret must be provided.", call. = FALSE)
  }
  if (!nzchar(experience_name)) {
    stop("experience_name must be provided.", call. = FALSE)
  }

  Sys.setenv(GLINT_TENANT_ID = tenant_id)
  Sys.setenv(GLINT_CLIENT_ID = client_id)
  Sys.setenv(GLINT_CLIENT_SECRET = client_secret)
  Sys.setenv(GLINT_EXPERIENCE_NAME = experience_name)

  message("Glint credentials saved to environment variables for this session.")

  if (isTRUE(save_to_renviron)) {
    renviron_path <- file.path(Sys.getenv("HOME"), ".Renviron")
    lines <- c(
      "",
      "# --- Viva Glint API credentials ---",
      paste0("GLINT_TENANT_ID=", tenant_id),
      paste0("GLINT_CLIENT_ID=", client_id),
      paste0("GLINT_CLIENT_SECRET=", client_secret),
      paste0("GLINT_EXPERIENCE_NAME=", experience_name)
    )
    cat(paste(lines, collapse = "\n"), "\n",
        file = renviron_path, append = TRUE)
        message("Also written to ", renviron_path,
          " - restart R to auto-load in future sessions.")
  }

  invisible(TRUE)
}


#' Read Viva Glint Survey Data via API
#'
#' Exports a survey cycle through the Microsoft Graph beta API, downloads the
#' resulting ZIP archive, and returns a \code{glint_survey} object. This is an
#' alternative to \code{read_glint_survey()} when you want to pull data directly
#' from Viva Glint instead of importing a local CSV export.
#'
#' @param survey_uuid Survey UUID from Viva Glint
#' @param cycle_id Survey cycle ID
#' @param emp_id_col Character string specifying the employee ID column name
#' @param first_name_col Column name for first name (default: "First Name")
#' @param last_name_col Column name for last name (default: "Last Name")
#' @param email_col Column name for email (default: "Email")
#' @param status_col Column name for status (default: "Status")
#' @param completion_date_col Column name for survey completion date
#'   (default: "Survey Cycle Completion Date")
#' @param sent_date_col Column name for survey sent date
#'   (default: "Survey Cycle Sent Date")
#' @param start_date Optional start date/time for the export window. Can be a
#'   character string in ISO 8601 format, or a Date/POSIXct value.
#' @param end_date Optional end date/time for the export window. Can be a
#'   character string in ISO 8601 format, or a Date/POSIXct value.
#' @param encoding Character string specifying file encoding (default: "UTF-8")
#' @param poll_interval Seconds to wait between status checks (default: 10)
#' @param max_attempts Maximum number of polling attempts (default: 60)
#' @param experience_name Optional Viva Glint experience name to override the
#'   GLINT_EXPERIENCE_NAME environment variable
#'
#' @return A \code{glint_survey} object (same structure as \code{read_glint_survey}).
#'   The \code{metadata$file_path} field is set to \code{NA}.
#'
#' @export
#'
#' @examples
#' \dontrun{
#' glint_setup(
#'   tenant_id = "your-tenant-id",
#'   client_id = "your-client-id",
#'   client_secret = "your-client-secret",
#'   experience_name = "your-experience-name"
#' )
#'
#' survey <- read_glint_survey_api(
#'   survey_uuid = "your-survey-uuid",
#'   cycle_id = "your-cycle-id",
#'   emp_id_col = "EMP ID"
#' )
#' }
read_glint_survey_api <- function(survey_uuid,
                                  cycle_id,
                                  emp_id_col = NULL,
                                  first_name_col = "First Name",
                                  last_name_col = "Last Name",
                                  email_col = "Email",
                                  status_col = "Status",
                                  completion_date_col = "Survey Cycle Completion Date",
                                  sent_date_col = "Survey Cycle Sent Date",
                                  start_date = NULL,
                                  end_date = NULL,
                                  encoding = "UTF-8",
                                  poll_interval = 10,
                                  max_attempts = 60,
                                  experience_name = NULL) {
  if (!nzchar(survey_uuid)) {
    stop("survey_uuid must be provided.", call. = FALSE)
  }
  if (!nzchar(cycle_id)) {
    stop("cycle_id must be provided.", call. = FALSE)
  }

  exp_name <- experience_name %||% glint_env(
    "GLINT_EXPERIENCE_NAME",
    "Experience Name"
  )

  export_url <- paste0(
    glint_graph_base, "/", utils::URLencode(exp_name, reserved = TRUE),
    "/surveys/", survey_uuid,
    "/surveyCycles/", cycle_id,
    "/exportSurveys"
  )

  data <- glint_run_export_pipeline(
    export_url,
    start_date = start_date,
    end_date = end_date,
    poll_interval = poll_interval,
    max_attempts = max_attempts,
    encoding = encoding,
    experience_name = exp_name
  )

  if (!is.data.frame(data)) {
    stop(
      "Export returned multiple CSV files with different schemas. ",
      "Please confirm the export content in Viva Glint and retry.",
      call. = FALSE
    )
  }

  build_glint_survey(
    data,
    emp_id_col,
    first_name_col = first_name_col,
    last_name_col = last_name_col,
    email_col = email_col,
    status_col = status_col,
    completion_date_col = completion_date_col,
    sent_date_col = sent_date_col,
    file_path = NA_character_
  )
}


glint_graph_base <- "https://graph.microsoft.com/beta/employeeExperience/sentiment/experiences"


#' Read a required Glint env var, stopping with a helpful message if missing.
#'
#' @keywords internal
glint_env <- function(var_name, label) {
  val <- Sys.getenv(var_name, unset = "")
  if (!nzchar(val)) {
    stop(
      label,
      " is not set. Run glint_setup() first to configure credentials.",
      call. = FALSE
    )
  }
  val
}


glint_format_datetime <- function(value) {
  if (is.null(value)) {
    return(NULL)
  }
  if (is.character(value) && !nzchar(value)) {
    return(NULL)
  }
  if (inherits(value, "Date")) {
    value <- as.POSIXct(value, tz = "UTC")
  }
  if (inherits(value, "POSIXt")) {
    return(format(as.POSIXct(value, tz = "UTC"), "%Y-%m-%dT%H:%M:%SZ"))
  }
  as.character(value)
}


glint_get_token <- function() {
  tid <- glint_env("GLINT_TENANT_ID", "Tenant ID")
  cid <- glint_env("GLINT_CLIENT_ID", "Client ID")
  csec <- glint_env("GLINT_CLIENT_SECRET", "Client Secret")

  url <- paste0(
    "https://login.microsoftonline.com/", tid, "/oauth2/v2.0/token"
  )

  resp <- httr::POST(
    url,
    body = list(
      client_id = cid,
      client_secret = csec,
      scope = "https://graph.microsoft.com/.default",
      grant_type = "client_credentials"
    ),
    encode = "form"
  )

  if (httr::http_error(resp)) {
    stop(
      "Token request failed (HTTP ", httr::status_code(resp), "): ",
      httr::content(resp, "text", encoding = "UTF-8"),
      call. = FALSE
    )
  }

  token_data <- httr::content(resp, "parsed", simplifyVector = TRUE)
  if (is.null(token_data$access_token)) {
    stop("Token response did not include an access token.", call. = FALSE)
  }

  expires_at <- as.numeric(Sys.time()) + token_data$expires_in
  Sys.setenv(GLINT_TOKEN = token_data$access_token)
  Sys.setenv(GLINT_TOKEN_EXPIRES = as.character(expires_at))

  invisible(token_data$access_token)
}


glint_ensure_token <- function() {
  token <- Sys.getenv("GLINT_TOKEN", unset = "")
  expires_at <- Sys.getenv("GLINT_TOKEN_EXPIRES", unset = "")

  if (nzchar(token) && nzchar(expires_at)) {
    remaining <- suppressWarnings(as.numeric(expires_at)) - as.numeric(Sys.time())
    if (!is.na(remaining) && remaining > 60) {
      return(token)
    }
  }

  glint_get_token()
  Sys.getenv("GLINT_TOKEN")
}


glint_start_export <- function(export_url,
                               start_date = NULL,
                               end_date = NULL) {
  token <- glint_ensure_token()
  body <- list()

  start_val <- glint_format_datetime(start_date)
  end_val <- glint_format_datetime(end_date)

  if (!is.null(start_val)) {
    body$startDateTime <- start_val
  }
  if (!is.null(end_val)) {
    body$endDateTime <- end_val
  }

  resp <- httr::POST(
    export_url,
    httr::add_headers(
      Authorization = paste("Bearer", token),
      `Content-Type` = "application/json"
    ),
    body = body,
    encode = "json"
  )

  if (httr::http_error(resp)) {
    stop(
      "Export request failed (HTTP ", httr::status_code(resp), "): ",
      httr::content(resp, "text", encoding = "UTF-8"),
      call. = FALSE
    )
  }

  result <- httr::content(resp, "parsed", simplifyVector = TRUE)
  if (is.null(result$id)) {
    stop("Export request did not return a job ID.", call. = FALSE)
  }

  message("Export job started - ID: ", result$id,
          if (!is.null(result$status)) paste0(" | Status: ", result$status) else "")
  result$id
}


glint_poll_status <- function(job_id,
                              experience_name,
                              poll_interval = 10,
                              max_attempts = 60) {
  status_url <- paste0(
    glint_graph_base, "/", utils::URLencode(experience_name, reserved = TRUE),
    "/operations('", job_id, "')"
  )

  for (i in seq_len(max_attempts)) {
    token <- glint_ensure_token()
    resp <- httr::GET(
      status_url,
      httr::add_headers(Authorization = paste("Bearer", token))
    )

    if (httr::http_error(resp)) {
      stop(
        "Status check failed (HTTP ", httr::status_code(resp), "): ",
        httr::content(resp, "text", encoding = "UTF-8"),
        call. = FALSE
      )
    }

    result <- httr::content(resp, "parsed", simplifyVector = TRUE)
    status <- tolower(result$status %||% "")

    if (status == "succeeded") {
      return(invisible(TRUE))
    }
    if (status == "failed") {
      stop("Export job failed: ", result$statusDetail %||% result$status,
           call. = FALSE)
    }

    Sys.sleep(poll_interval)
  }

  stop("Timed out after ", max_attempts * poll_interval, " seconds.", call. = FALSE)
}


glint_download_export <- function(job_id, experience_name) {
  token <- glint_ensure_token()
  download_url <- paste0(
    glint_graph_base, "/", utils::URLencode(experience_name, reserved = TRUE),
    "/operations('", job_id, "')/content"
  )

  resp <- httr::GET(
    download_url,
    httr::add_headers(Authorization = paste("Bearer", token))
  )

  if (httr::http_error(resp)) {
    stop(
      "Download failed (HTTP ", httr::status_code(resp), "): ",
      httr::content(resp, "text", encoding = "UTF-8"),
      call. = FALSE
    )
  }

  resp
}


glint_import_export_zip <- function(resp, encoding = "UTF-8") {
  raw_bytes <- httr::content(resp, "raw")

  tmp_zip <- tempfile(fileext = ".zip")
  tmp_dir <- tempfile("glint_export_")

  on.exit({
    if (file.exists(tmp_zip)) {
      unlink(tmp_zip)
    }
    if (dir.exists(tmp_dir)) {
      unlink(tmp_dir, recursive = TRUE)
    }
  }, add = TRUE)

  writeBin(raw_bytes, tmp_zip)
  dir.create(tmp_dir)
  utils::unzip(tmp_zip, exdir = tmp_dir)

  files <- list.files(tmp_dir, pattern = "\\.csv$",
                      recursive = TRUE, full.names = TRUE)
  if (length(files) == 0) {
    stop("No CSV files found in the ZIP archive.", call. = FALSE)
  }

  dfs <- lapply(files, function(f) {
    readr::read_csv(
      f,
      locale = readr::locale(encoding = encoding),
      show_col_types = FALSE
    )
  })
  names(dfs) <- tools::file_path_sans_ext(basename(files))

  if (length(dfs) == 1) {
    return(dfs[[1]])
  }

  first_cols <- names(dfs[[1]])
  same_schema <- all(vapply(dfs, function(df) {
    identical(names(df), first_cols)
  }, logical(1)))

  if (same_schema) {
    return(dplyr::bind_rows(dfs))
  }

  dfs
}


glint_run_export_pipeline <- function(export_url,
                                      start_date = NULL,
                                      end_date = NULL,
                                      poll_interval = 10,
                                      max_attempts = 60,
                                      encoding = "UTF-8",
                                      experience_name) {
  job_id <- glint_start_export(export_url, start_date = start_date, end_date = end_date)
  glint_poll_status(
    job_id,
    experience_name = experience_name,
    poll_interval = poll_interval,
    max_attempts = max_attempts
  )
  resp <- glint_download_export(job_id, experience_name = experience_name)
  glint_import_export_zip(resp, encoding = encoding)
}

#' @importFrom dplyr %>%
#' @importFrom stats cor cor.test sd setNames
#' @keywords internal
"_PACKAGE"

utils::globalVariables(c(
	".data",
	"First Name",
	"Last Name",
	"comment",
	"comment_topics",
	"communality",
	"correlation",
	"cycle",
	"factor",
	"glint_score",
	"group_size",
	"loading",
	"loading_label",
	"manager_id",
	"manager_name",
	"n",
	"p_value",
	"pct",
	"question",
	"question1",
	"question2",
	"response",
	"sensitive_flag",
	"sentiment",
	"team_size",
	"value"
))

## usethis namespace: start
## usethis namespace: end
NULL

#' Aggregate Responses by Manager
#'
#' Rolls up survey responses to the manager level, calculating the same
#' metrics as summarize_survey() for each manager's team.
#'
#' @param survey A glint_survey object or data frame containing survey data
#' @param scale_points Integer specifying the number of scale points (2-11)
#' @param emp_id_col Character string specifying the employee ID column name
#' @param manager_id_col Character string specifying the manager ID column name
#' @param full_tree Logical indicating whether to include full subtree
#'   (all indirect reports) or only direct reports (default: FALSE)
#' @param plot Logical. If \code{TRUE}, prints a ranked lollipop chart of team
#'   Glint Scores by manager and returns the data invisibly. When multiple
#'   questions are present the chart is faceted by question. Requires
#'   \pkg{ggplot2}. Default: \code{FALSE}.
#'
#' @return A tibble with one row per manager-question combination containing:
#'   \describe{
#'     \item{manager_id}{The manager's employee ID}
#'     \item{manager_name}{The manager's full name (First Name + Last Name)}
#'     \item{question}{The question text}
#'     \item{team_size}{Number of employees in the team (direct reports only, or
#'       full subtree when \code{full_tree = TRUE})}
#'     \item{mean, sd, glint_score, n_responses, n_skips, n_total}{Descriptive
#'       statistics for this manager's team on this question}
#'     \item{pct_favorable, pct_neutral, pct_unfavorable}{Favorability
#'       percentages for this manager's team on this question}
#'   }
#'   When \code{plot = TRUE}, the same tibble is returned invisibly after
#'   printing the plot.
#'
#' @export
#'
#' @examples
#' survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
#' survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
#'
#' # Direct reports only
#' manager_summary <- aggregate_by_manager(survey, scale_points = 5,
#'                                         emp_id_col = "EMP ID",
#'                                         manager_id_col = "Manager ID")
#'
#' # Full organizational tree
#' manager_summary_full <- aggregate_by_manager(survey, scale_points = 5,
#'                                              emp_id_col = "EMP ID",
#'                                              manager_id_col = "Manager ID",
#'                                              full_tree = TRUE)
#'
#' # With ranked dot plot
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   aggregate_by_manager(survey, scale_points = 5, emp_id_col = "EMP ID",
#'                        manager_id_col = "Manager ID", plot = TRUE)
#' }
aggregate_by_manager <- function(survey, scale_points, emp_id_col = NULL,
                                 manager_id_col, full_tree = FALSE,
                                 plot = FALSE) {
  if (plot) .check_ggplot2()
  if (inherits(survey, "glint_survey")) {
    emp_id_col <- emp_id_col %||% survey$metadata$emp_id_col
    data <- survey$data
    questions <- survey$metadata$questions
    col_map <- survey$metadata$standard_column_map %||% list()
    first_name_col <- col_map$first_name %||% "First Name"
    last_name_col <- col_map$last_name %||% "Last Name"
  } else {
    data <- survey
    questions <- extract_questions(data)
    first_name_col <- "First Name"
    last_name_col <- "Last Name"
  }

  if (is.null(emp_id_col)) {
    stop("emp_id_col must be specified when survey is a plain data frame")
  }

  missing_name_labels <- character(0)
  if (is.null(first_name_col) || !nzchar(first_name_col)) {
    missing_name_labels <- c(missing_name_labels, "First Name")
  }
  if (is.null(last_name_col) || !nzchar(last_name_col)) {
    missing_name_labels <- c(missing_name_labels, "Last Name")
  }
  if (length(missing_name_labels) > 0) {
    stop(
      "aggregate_by_manager() requires column name(s) for: ",
      paste(missing_name_labels, collapse = ", "),
      ". Provide first_name_col/last_name_col when reading the survey.",
      call. = FALSE
    )
  }
  missing_name_cols <- setdiff(c(first_name_col, last_name_col), names(data))
  if (length(missing_name_cols) > 0) {
    stop(
      "aggregate_by_manager() requires the following column(s) to build manager names: ",
      paste0("'", missing_name_cols, "'", collapse = ", "),
      "\n\nColumns detected in the data frame:\n",
      paste0("  - ", names(data), collapse = "\n"),
      "\n\nTo resolve, pass first_name_col/last_name_col when reading with read_glint_survey() or read_glint_survey_api(), or rename your columns.",
      call. = FALSE
    )
  }

  managers <- data %>%
    dplyr::filter(!is.na(.data[[manager_id_col]])) %>%
    dplyr::distinct(dplyr::across(dplyr::all_of(manager_id_col))) %>%
    dplyr::pull(manager_id_col)

  manager_names <- data %>%
    dplyr::filter(.data[[emp_id_col]] %in% managers) %>%
    dplyr::distinct(dplyr::across(dplyr::all_of(c(emp_id_col, first_name_col, last_name_col)))) %>%
    dplyr::mutate(manager_name = paste(.data[[first_name_col]], .data[[last_name_col]]))

  results <- purrr::map_dfr(managers, function(mgr_id) {
    if (full_tree) {
      team_members <- get_all_reports(mgr_id, data, emp_id_col, manager_id_col)
    } else {
      team_members <- data %>%
        dplyr::filter(.data[[manager_id_col]] == mgr_id) %>%
        dplyr::pull(emp_id_col)
    }

    if (length(team_members) == 0) {
      return(NULL)
    }

    team_data <- data %>%
      dplyr::filter(.data[[emp_id_col]] %in% team_members)

    question_results <- summarize_survey(team_data, scale_points = scale_points, questions = "all", emp_id_col = emp_id_col)
    question_results$manager_id <- mgr_id
    question_results$team_size <- length(team_members)

    return(question_results)
  })

  results <- results %>%
    dplyr::left_join(manager_names, by = setNames(emp_id_col, "manager_id")) %>%
    dplyr::select(manager_id, manager_name, question, team_size, dplyr::everything())

  if (plot) {
    print(.plot_manager(results))
    return(invisible(results))
  }

  return(results)
}


#' Get All Reports for a Manager
#'
#' Internal function to get all direct and indirect reports for a manager.
#'
#' @param manager_id Character string of manager's employee ID
#' @param data Data frame with survey data
#' @param emp_id_col Character string specifying the employee ID column name
#' @param manager_id_col Character string specifying the manager ID column name
#'
#' @return Character vector of employee IDs for all reports
#'
#' @keywords internal
get_all_reports <- function(manager_id, data, emp_id_col, manager_id_col) {
  direct_reports <- data %>%
    dplyr::filter(.data[[manager_id_col]] == manager_id) %>%
    dplyr::pull(emp_id_col) %>%
    unique()

  if (length(direct_reports) == 0) {
    return(character(0))
  }

  all_reports <- direct_reports

  # Recursively get reports of reports
  for (report in direct_reports) {
    indirect_reports <- get_all_reports(report, data, emp_id_col, manager_id_col)
    all_reports <- c(all_reports, indirect_reports)
  }

  return(unique(all_reports))
}

---
title: "Introduction to vivaglint"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to vivaglint}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Overview

**vivaglint** is an R package built for HR analysts who work with Microsoft Viva Glint survey exports. It handles the repetitive data wrangling that Glint's native UI doesn't support — multi-cycle trend analysis, manager roll-ups, demographic segmentation, attrition risk scoring, and comment search — so you can spend more time interpreting results and less time reshaping data.

## Key Capabilities

- **Data Import & Validation**: Reads Glint CSV exports and validates the structure automatically
- **Summary Statistics**: Mean, SD, Glint Score (0–100), response counts, favorability percentages
- **Multi-Cycle Trending**: Compare engagement scores across survey waves
- **Manager Roll-Ups**: Aggregate results by direct or full reporting tree
- **Demographic Segmentation**: Slice results by any employee attribute (department, gender, tenure, etc.)
- **Attrition Risk Analysis**: Link survey scores to actual turnover outcomes
- **Comment Search**: Full-text search across all comment columns at once

## Getting Started

### Installation

```{r eval=FALSE}
devtools::install_github("microsoft/vivaglint")
library(vivaglint)
```

### Basic Workflow

#### 1. Import Your Glint Export

Export your survey from Viva Glint as a CSV and load it with `read_glint_survey()`:

```{r eval=FALSE}
survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
```

If you prefer to pull data directly from the Viva Glint API, configure
credentials once and use `read_glint_survey_api()`:

```{r eval=FALSE}
glint_setup(
  tenant_id = "your-tenant-id",
  client_id = "your-client-id",
  client_secret = "your-client-secret",
  experience_name = "your-experience-name"
)
survey <- read_glint_survey_api(
  survey_uuid = "your-survey-uuid",
  cycle_id = "your-cycle-id",
  emp_id_col = "EMP ID"
)
```

This returns a `glint_survey` object containing:
- **data**: The full survey response table
- **metadata**: Question names, column mappings, and respondent counts

#### 2. Get a Question-Level Summary

Calculate metrics across all survey questions in one call:

```{r eval=FALSE}
summary <- summarize_survey(survey, scale_points = 5)
```

Each row in the output represents one question and includes:
- `mean` — Average response on the raw scale
- `sd` — Standard deviation
- `glint_score` — Score transformed to 0–100, matching what appears in the Viva Glint UI
- `n_responses` — Number of employees who answered the question
- `n_skips` — Number of employees who skipped the question
- `n_total` — Total respondents
- `pct_favorable` — Percentage of favorable responses
- `pct_neutral` — Percentage of neutral responses
- `pct_unfavorable` — Percentage of unfavorable responses

**About Glint Score**: The Glint Score is calculated as `round(((mean - 1) / (scale_points - 1)) * 100)`, placing every question on a common 0–100 scale regardless of the original scale format.

**About Favorability**: The package uses Viva Glint's standard favorability thresholds for each scale type. On a 5-point scale, for example, 4–5 is favorable, 3 is neutral, and 1–2 is unfavorable.

#### 3. Focus on Specific Questions

If you're presenting to a leadership team or investigating a particular theme, filter to just the questions you care about:

```{r eval=FALSE}
# Engagement-specific questions
engagement_qs <- c(
  "I would recommend my team as a great place to work",
  "My work is meaningful"
)
engagement_summary <- summarize_survey(survey,
                                       scale_points = 5,
                                       questions = engagement_qs)
```

#### 4. Explore Response Distributions

See exactly how many employees chose each response value — useful when a mean alone doesn't tell the full story:

```{r eval=FALSE}
distributions <- get_response_dist(survey, scale_points = 5)
```

The output adds columns like `count_1`, `count_2`, `pct_1`, `pct_2`, etc. for each response value.

---

## Multi-Cycle Trend Analysis

One of the most common HR reporting tasks is tracking whether engagement improved since the last survey. `compare_cycles()` takes multiple survey objects and aligns them by question:

```{r eval=FALSE}
survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
survey_q1 <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
survey_q2 <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
survey_q3 <- read_glint_survey(survey_path, emp_id_col = "EMP ID")

trends <- compare_cycles(
  survey_q1, survey_q2, survey_q3,
  scale_points = 5,
  cycle_names = c("Q1 FY25", "Q2 FY25", "Q3 FY25")
)
```

The output includes all metrics from `summarize_survey()` for each cycle, plus:
- `change_from_previous` — Point change in mean score vs. prior cycle
- `pct_change_from_previous` — Percentage change vs. prior cycle

This is the foundation for executive trend slides: which items are improving, holding steady, or declining quarter over quarter.

---

## Manager-Level Analysis

HR business partners often need to understand which managers' teams are scoring below average, or identify pockets of high engagement to learn from.

### Roll Up to Manager Level

```{r eval=FALSE}
# Direct reports only
manager_summary <- aggregate_by_manager(survey, scale_points = 5)

# Full org tree (includes indirect reports)
manager_full <- aggregate_by_manager(survey, scale_points = 5, full_tree = TRUE)
```

Each row represents one manager × question combination and includes:
- `manager_id`, `manager_name`
- `team_size`
- All standard metrics: `mean`, `sd`, `glint_score`, `n_responses`, `n_skips`, `n_total`, `pct_favorable`, `pct_neutral`, `pct_unfavorable`

You can filter the result to identify managers with low favorability on a specific question:

```{r eval=FALSE}
library(dplyr)

# Managers where fewer than 50% of their team is favorable on a key item
low_engagement_managers <- manager_summary %>%
  filter(question == "I would recommend my team as a great place to work",
         pct_favorable < 50) %>%
  arrange(pct_favorable)
```

---

## Demographic Analysis

### Segment by Employee Attributes

`analyze_by_attributes()` lets you break survey results down by any combination of employee attributes — department, gender, tenure group, location, job level, etc.

```{r eval=FALSE}
attr_path <- system.file("extdata", "employee_attributes.csv", package = "vivaglint")
demo_results <- analyze_by_attributes(
  survey,
  attribute_file = attr_path,
  scale_points = 5,
  attribute_cols = c("Department", "Gender", "Tenure Group"),
  min_group_size = 10  # Suppress groups below this size for privacy
)
```

This is useful for identifying which employee populations have systematically lower scores and on which questions.

### Pre-Joining Attributes for Multiple Analyses

If you plan to run several analyses against the same attribute file, join it once and reuse the enriched survey object:

```{r eval=FALSE}
# Join once
attr_path <- system.file("extdata", "employee_attributes.csv", package = "vivaglint")
survey_enriched <- join_attributes(survey, attr_path)

# Reuse for multiple analyses — no need to re-read the file
dept_results <- analyze_by_attributes(
  survey_enriched,
  scale_points = 5,
  attribute_cols = "Department"
)

gender_results <- analyze_by_attributes(
  survey_enriched,
  scale_points = 5,
  attribute_cols = "Gender"
)

# Filter to a subpopulation before analyzing
na_only <- survey_enriched
na_only$data <- filter(survey_enriched$data, Region == "North America")
na_results <- analyze_by_attributes(na_only, scale_points = 5,
                                    attribute_cols = "Department")
```

Attributes are stored in `survey$metadata$attribute_cols` after joining, so they are excluded from question detection in downstream analyses.

---

## Attrition Risk Analysis

Linking engagement scores to actual turnover is a high-value analysis for HR leaders. `analyze_attrition()` computes attrition rates by favorability group — showing whether employees who scored low on engagement were more likely to leave.

```{r eval=FALSE}
# Basic attrition analysis (90, 180, and 365 days post-survey)
attrition_path <- system.file("extdata", "attrition.csv", package = "vivaglint")
attrition <- analyze_attrition(
  survey,
  attrition_file = attrition_path,
  emp_id_col = "EMP ID",
  term_date_col = "Termination Date",
  scale_points = 5
)
```

The output shows, for each question and time period, the attrition rates for favorable vs. unfavorable responders along with a risk ratio — making it straightforward to identify which survey items are the strongest leading indicators of turnover.

### Segment Attrition by Demographics

Combine attrition analysis with employee attributes to answer questions like "Are unfavorable responders in Engineering leaving at higher rates than those in Sales?":

```{r eval=FALSE}
attr_path <- system.file("extdata", "employee_attributes.csv", package = "vivaglint")
attrition_path <- system.file("extdata", "attrition.csv", package = "vivaglint")
survey_enriched <- join_attributes(survey, attr_path)

attrition_by_dept <- analyze_attrition(
  survey_enriched,
  attrition_file = attrition_path,
  emp_id_col = "EMP ID",
  term_date_col = "Termination Date",
  scale_points = 5,
  attribute_cols = c("Department", "Job Level"),
  min_group_size = 10
)
```

---

## Correlation and Factor Analysis

### Understand Which Items Move Together

Correlation analysis is useful for identifying clusters of related questions — often as a first step before building a composite score or validating that a set of items measures a single construct:

```{r eval=FALSE}
# Pearson correlations in long format (default)
correlations <- get_correlations(survey)

# Spearman correlations (more robust for ordinal scale data)
correlations_spearman <- get_correlations(survey, method = "spearman")

# Correlation matrix
cor_matrix <- get_correlations(survey, format = "matrix")
```

Supported methods: `"pearson"` (default), `"spearman"`, `"kendall"`

### Factor Analysis

Factor analysis identifies the latent constructs underlying a set of survey items. This can validate whether your "manager effectiveness" items truly cluster together, or reveal unexpected groupings:

```{r eval=FALSE}
# Requires the psych package
factors <- extract_survey_factors(survey, n_factors = 3, rotation = "oblimin")

# Consolidated summary: item, factor assignment, loading, label, communality
print(factors$factor_summary)

# Filter to items with strong factor loadings only
strong_loaders <- dplyr::filter(factors$factor_summary, loading_label == "Strong")

# Access the raw psych object for advanced use
factors$fa_object
```

Loading labels: **Strong** (≥ 0.75), **Medium** (0.60–0.74), **Weak** (< 0.60)

---

## Working with Comments

### Full-Text Comment Search

Search across all comment columns at once — useful for surfacing themes around specific topics like "flexibility", "burnout", or a manager's name:

```{r eval=FALSE}
# Fuzzy search (default) — tolerates minor spelling differences
flexibility_comments <- search_comments(survey, "flexibility")

# Exact, case-sensitive match
exact_results <- search_comments(survey, "work from home", exact = TRUE)

# Broaden fuzzy tolerance to catch more spelling variation
results <- search_comments(survey, "colaboration", max_distance = 0.3)
```

Each result row includes:
- `question` — Which question the comment was attached to
- `response` — The numeric score the employee gave
- `comment` — The comment text
- `topics` — Topic tags assigned by Glint

### Convert to Long Format for NLP

To route comments into a text analysis or NLP pipeline, reshape the survey to long format and filter to rows with comments:

```{r eval=FALSE}
# All responses in long format
long_all <- pivot_long(survey, data_type = "all")

# Comments only
long_comments <- pivot_long(survey, data_type = "comments")

# Both as separate tibbles
both <- pivot_long(survey, data_type = "both")
comments_df <- both$comments
```

---

## Separating Quantitative and Qualitative Data

For workflows that route numeric scores and open-text comments to different pipelines (e.g., numeric data to a statistical model, comments to an LLM), use `split_survey_data()`:

```{r eval=FALSE}
parts <- split_survey_data(survey)

# Numeric scores only — standard respondent columns + one score column per question
quantitative <- parts$quantitative

# Comments only — EMP ID + all _COMMENT, _COMMENT_TOPICS, _SENSITIVE_COMMENT_FLAG columns
qualitative <- parts$qualitative

# Pass numeric data directly to vivaglint functions
summary <- summarize_survey(parts$quantitative, scale_points = 5,
                            emp_id_col = "EMP ID")

# Rejoin at any time using EMP ID
full_data <- dplyr::left_join(parts$quantitative, parts$qualitative, by = "EMP ID")
```

---

## Privacy and Data Handling

### Minimum Group Sizes

Use `min_group_size` wherever it's available to suppress results for groups that are too small to protect individual anonymity:

```{r eval=FALSE}
# Default is 5; consider 10 or higher for sensitive analyses
attr_path <- system.file("extdata", "employee_attributes.csv", package = "vivaglint")
demo_results <- analyze_by_attributes(
  survey,
  attribute_file = attr_path,
  scale_points = 5,
  attribute_cols = c("Department", "Gender"),
  min_group_size = 10
)
```

### Local Processing

This package processes all data locally within your R environment. No employee data is transmitted to any external service, including Microsoft. Always follow your organization's data handling and privacy policies when working with employee survey data.

---

## Additional Resources

- **Function Documentation**: `?read_glint_survey`, `?summarize_survey`, `?analyze_by_attributes`, etc.
- **GitHub**: https://github.com/microsoft/vivaglint
- **Issues**: https://github.com/microsoft/vivaglint/issues

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_question_stem}
\alias{get_question_stem}
\title{Get Question Stem from Column Name}
\usage{
get_question_stem(col_name)
}
\arguments{
\item{col_name}{Character string representing a column name}
}
\value{
Character string with the base question text
}
\description{
Extracts the base question text from a column name by removing
standard suffixes (_COMMENT, _COMMENT_TOPICS, _SENSITIVE_COMMENT_FLAG).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{validate_glint_structure}
\alias{validate_glint_structure}
\title{Validate Glint Data Structure}
\usage{
validate_glint_structure(
  data,
  emp_id_col,
  first_name_col = "First Name",
  last_name_col = "Last Name",
  email_col = "Email",
  status_col = "Status",
  completion_date_col = "Survey Cycle Completion Date",
  sent_date_col = "Survey Cycle Sent Date"
)
}
\arguments{
\item{data}{A data frame to validate}

\item{emp_id_col}{Character string specifying the employee ID column name}

\item{first_name_col}{Column name for first name}

\item{last_name_col}{Column name for last name}

\item{email_col}{Column name for email}

\item{status_col}{Column name for status}

\item{completion_date_col}{Column name for survey completion date}

\item{sent_date_col}{Column name for survey sent date}
}
\value{
NULL (invisibly) if validation passes, otherwise throws an error
}
\description{
Internal function to validate that a data frame conforms to the expected
Viva Glint export structure. Throws detailed, human-readable errors if validation fails.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{build_glint_survey}
\alias{build_glint_survey}
\title{Build a glint_survey Object}
\usage{
build_glint_survey(
  data,
  emp_id_col,
  first_name_col = "First Name",
  last_name_col = "Last Name",
  email_col = "Email",
  status_col = "Status",
  completion_date_col = "Survey Cycle Completion Date",
  sent_date_col = "Survey Cycle Sent Date",
  file_path = NA_character_
)
}
\arguments{
\item{data}{A data frame containing Viva Glint survey data}

\item{emp_id_col}{Character string specifying the employee ID column name}

\item{first_name_col}{Column name for first name}

\item{last_name_col}{Column name for last name}

\item{email_col}{Column name for email}

\item{status_col}{Column name for status}

\item{completion_date_col}{Column name for survey completion date}

\item{sent_date_col}{Column name for survey sent date}

\item{file_path}{Character string for the source file path (or NA when not
loaded from disk)}
}
\value{
A \code{glint_survey} object
}
\description{
Internal helper to validate, parse, and attach metadata.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyze.R
\name{get_response_dist}
\alias{get_response_dist}
\title{Get Response Distribution}
\usage{
get_response_dist(survey, questions = "all", plot = FALSE)
}
\arguments{
\item{survey}{A glint_survey object or data frame containing survey data}

\item{questions}{Character vector of question text(s) to analyze, or "all" to
analyze all questions (default: "all")}

\item{plot}{Logical. If \code{TRUE}, prints a stacked bar chart of response
value distributions (red \eqn{\to} blue gradient) and returns the data
invisibly. Requires \pkg{ggplot2}. Default: \code{FALSE}.}
}
\value{
A tibble with one row per question containing:
\describe{
\item{question}{The question text}
\item{count_X}{Count of responses with value X (for each unique response value)}
\item{pct_X}{Percentage of responses with value X (for each unique response value)}
}
When \code{plot = TRUE}, the same tibble is returned invisibly after
printing the plot.
}
\description{
Calculates the distribution of response values for survey questions, returning
counts and percentages for each response value in tidy format.
}
\examples{
survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")

# Get response distribution for all questions
dist <- get_response_dist(survey)

# Get response distribution for specific questions
dist_subset <- get_response_dist(survey,
  questions = c("My work is meaningful", "I feel valued"))

# With distribution chart
if (requireNamespace("ggplot2", quietly = TRUE)) {
  get_response_dist(survey, plot = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{read_glint_survey}
\alias{read_glint_survey}
\title{Read Viva Glint Survey Data}
\usage{
read_glint_survey(
  file_path,
  emp_id_col = NULL,
  first_name_col = "First Name",
  last_name_col = "Last Name",
  email_col = "Email",
  status_col = "Status",
  completion_date_col = "Survey Cycle Completion Date",
  sent_date_col = "Survey Cycle Sent Date",
  encoding = "UTF-8"
)
}
\arguments{
\item{file_path}{Character string specifying the path to the CSV file}

\item{emp_id_col}{Character string specifying the name of the employee ID
column in the survey export (e.g., \code{"Employee ID"},
\code{"EmployeeID"}, \code{"emp_id"}). The column name varies by
organization. The value is stored in \code{survey$metadata$emp_id_col}
and used automatically by downstream functions so you do not need to
repeat it on every call.}

\item{first_name_col}{Column name for first name (default: "First Name")}

\item{last_name_col}{Column name for last name (default: "Last Name")}

\item{email_col}{Column name for email (default: "Email")}

\item{status_col}{Column name for status (default: "Status")}

\item{completion_date_col}{Column name for survey completion date
(default: "Survey Cycle Completion Date")}

\item{sent_date_col}{Column name for survey sent date
(default: "Survey Cycle Sent Date")}

\item{encoding}{Character string specifying file encoding (default: "UTF-8")}
}
\value{
A \code{glint_survey} object (an S3 class extending \code{list}) with
two elements:
\describe{
\item{data}{A tibble containing all survey responses. Date columns are
parsed automatically from DD-MM-YYYY HH:MM format to POSIXct.}
\item{metadata}{A list with five elements: \code{standard_columns} (the
eight standard Glint column names), \code{questions} (a tibble from
\code{extract_questions()} with one row per question), \code{n_respondents}
(total row count), \code{n_questions} (number of questions), and
\code{file_path} (the path originally supplied).}
}
}
\description{
Reads a Viva Glint survey export CSV file with automatic validation and parsing.
This function validates the file structure, parses dates, and organizes the data
into a structured format ready for analysis.
To import directly from the Viva Glint API, use
\code{read_glint_survey_api()}.
}
\examples{
survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
head(survey$data)
survey$metadata$questions
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{extract_questions}
\alias{extract_questions}
\title{Extract Questions from Glint Survey}
\usage{
extract_questions(data, emp_id_col = NULL)
}
\arguments{
\item{data}{A data frame or glint_survey object containing survey data}

\item{emp_id_col}{Character string specifying the employee ID column name.
When \code{data} is a \code{glint_survey} object this is resolved
automatically from \code{data$metadata$emp_id_col}. Required when
\code{data} is a plain data frame.}
}
\value{
A tibble with one row per question containing:
\describe{
\item{question}{The question text}
\item{response_col}{Column name for numeric responses}
\item{comment_col}{Column name for comments}
\item{topics_col}{Column name for comment topics}
\item{flag_col}{Column name for sensitive comment flags}
}
}
\description{
Parses column names to extract unique questions and their associated column names.
}
\examples{
survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
questions <- extract_questions(survey)
print(questions)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyze.R
\name{expand_value_distributions}
\alias{expand_value_distributions}
\title{Expand Value Distributions}
\usage{
expand_value_distributions(analysis_df)
}
\arguments{
\item{analysis_df}{A tibble from get_response_dist() with value_counts and value_percents list columns}
}
\value{
A tibble with expanded columns for each response value
}
\description{
Internal function to expand value_counts and value_percents list columns
into separate columns for each response value.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyze.R
\name{search_comments}
\alias{search_comments}
\title{Search Survey Comments}
\usage{
search_comments(survey, query, exact = FALSE, max_distance = 0.2)
}
\arguments{
\item{survey}{A glint_survey object or data frame containing survey data}

\item{query}{Character string to search for within comments}

\item{exact}{Logical. If \code{TRUE}, performs case-sensitive exact substring
matching. If \code{FALSE} (default), performs case-insensitive approximate
matching that also tolerates minor spelling differences.}

\item{max_distance}{Numeric between 0 and 1 controlling fuzzy match tolerance
when \code{exact = FALSE}. Higher values allow more differences (e.g. more
typos). Default is 0.2.}
}
\value{
A tibble with one row per matching comment containing:
\describe{
\item{question}{The survey question text}
\item{response}{The numeric survey response value}
\item{comment}{The comment text that matched the query}
\item{topics}{The comment topic(s) associated with the comment}
}
Returns an empty tibble with the same columns if no matches are found.
}
\description{
Searches through all survey comment text and returns matching responses
with their associated question text, numeric response values, comments, and
topics. Supports both exact substring matching and fuzzy (approximate)
matching that tolerates minor spelling differences.
}
\examples{
survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")

# Fuzzy search (default) - finds "manager", "Manager", "managers", etc.
results <- search_comments(survey, "manager")

# Exact search - finds only the literal string "Manager" (case-sensitive)
results_exact <- search_comments(survey, "Manager", exact = TRUE)

# Wider fuzzy tolerance to catch more spelling variations
results_wide <- search_comments(survey, "management", max_distance = 0.3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyze.R
\name{compare_cycles}
\alias{compare_cycles}
\title{Compare Survey Cycles}
\usage{
compare_cycles(..., scale_points, cycle_names = NULL, plot = FALSE)
}
\arguments{
\item{...}{Two or more glint_survey objects or data frames to compare}

\item{scale_points}{Integer specifying the number of scale points (2-11)}

\item{cycle_names}{Optional character vector of names for each cycle.
If not provided, will use "Cycle 1", "Cycle 2", etc.}

\item{plot}{Logical. If \code{TRUE}, prints a line chart of Glint Score over
cycles (one line per question) and returns the data invisibly. Requires
\pkg{ggplot2}. Default: \code{FALSE}.}
}
\value{
A tibble with one row per question-cycle combination containing:
\describe{
\item{cycle}{The cycle name or number}
\item{question}{The question text}
\item{mean, sd, n_responses, n_skips, n_total}{Descriptive
statistics for this question in this cycle (see \code{summarize_survey()})}
\item{pct_favorable, pct_neutral, pct_unfavorable}{Favorability
percentages for this question in this cycle}
\item{change_from_previous}{Change in raw mean score from the previous
cycle (NA for the first cycle)}
\item{pct_change_from_previous}{Percentage change in raw mean score from
the previous cycle (NA for the first cycle)}
\item{glint_score_change_from_previous}{Change in Glint Score (0-100
scale) from the previous cycle (NA for the first cycle)}
}
When \code{plot = TRUE}, the same tibble is returned invisibly after
printing the plot.
}
\description{
Compares question-level metrics across multiple survey cycles, calculating
change scores and trends over time.
}
\examples{
survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
survey1 <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
survey2 <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
survey3 <- read_glint_survey(survey_path, emp_id_col = "EMP ID")

comparison <- compare_cycles(survey1, survey2, survey3,
                              scale_points = 5,
                              cycle_names = c("Q1 2024", "Q2 2024", "Q3 2024"))
print(comparison)

# With trend chart
if (requireNamespace("ggplot2", quietly = TRUE)) {
  compare_cycles(survey1, survey2, survey3, scale_points = 5,
                 cycle_names = c("Q1 2024", "Q2 2024", "Q3 2024"),
                 plot = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyze.R
\name{analyze_attrition}
\alias{analyze_attrition}
\title{Analyze Employee Attrition}
\usage{
analyze_attrition(
  survey,
  attrition_file,
  emp_id_col = NULL,
  term_date_col,
  scale_points,
  time_periods = c(90, 180, 365),
  attribute_cols = NULL,
  min_group_size = 5,
  plot = FALSE
)
}
\arguments{
\item{survey}{A glint_survey object or data frame containing survey data}

\item{attrition_file}{Path to CSV file containing employee attrition data}

\item{emp_id_col}{Character string specifying the column name for employee ID}

\item{term_date_col}{Character string specifying the column name for termination date}

\item{scale_points}{Integer specifying the number of scale points (2-11)}

\item{time_periods}{Integer vector specifying time periods in days to analyze
(default: c(90, 180, 365))}

\item{attribute_cols}{Optional character vector of attribute column names to
segment results by (e.g., \code{c("Department", "Gender")}). Attributes must
already be joined to the survey via \code{join_attributes()}. When \code{NULL}
(default), results are returned for the overall population.}

\item{min_group_size}{Minimum number of employees required in an attribute group
for it to be included in results (default: 5). Ignored when
\code{attribute_cols = NULL}.}

\item{plot}{Logical. If \code{TRUE}, prints a grouped bar chart of favorable
vs. unfavorable attrition rates faceted by time period and returns the data
invisibly. When \code{attribute_cols} is supplied, the first attribute
column is used as an additional facet dimension. Requires \pkg{ggplot2}.
Default: \code{FALSE}.}
}
\value{
A tibble with one row per (attribute group)-question-time period
combination containing:
\describe{
\item{attribute columns}{One column per entry in \code{attribute_cols}
(only present when \code{attribute_cols} is supplied)}
\item{group_size}{Number of employees in the attribute group (only present
when \code{attribute_cols} is supplied)}
\item{question}{The question text}
\item{days}{The time period in days}
\item{favorable_n}{Number of employees who responded favorably}
\item{favorable_attrition}{Proportion who left within time period (favorable)}
\item{unfavorable_n}{Number of employees who responded unfavorably}
\item{unfavorable_attrition}{Proportion who left within time period (unfavorable)}
\item{attrition_ratio}{Ratio of unfavorable to favorable attrition rates}
}
When \code{plot = TRUE}, the same tibble is returned invisibly after
printing the plot.
}
\description{
Analyzes the relationship between survey responses and employee attrition,
calculating how much more (or less) likely employees are to leave within
specified time periods if they respond unfavorably vs favorably to survey questions.
}
\examples{
survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
attrition_file <- system.file("extdata", "attrition.csv", package = "vivaglint")
attr_path <- system.file("extdata", "employee_attributes.csv", package = "vivaglint")
survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")

# Overall attrition analysis
attrition <- analyze_attrition(
  survey,
  attrition_file = attrition_file,
  emp_id_col = "EMP ID",
  term_date_col = "Termination Date",
  scale_points = 5
)

# Attrition segmented by Department and Gender
survey_enriched <- join_attributes(survey, attr_path, emp_id_col = "EMP ID")
attrition_by_dept <- analyze_attrition(
  survey_enriched,
  attrition_file = attrition_file,
  emp_id_col = "EMP ID",
  term_date_col = "Termination Date",
  scale_points = 5,
  attribute_cols = c("Department", "Gender"),
  min_group_size = 2
)

# With attrition chart
if (requireNamespace("ggplot2", quietly = TRUE)) {
  analyze_attrition(survey, attrition_file = attrition_file,
                    emp_id_col = "EMP ID", term_date_col = "Termination Date",
                    scale_points = 5, plot = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api.R
\name{read_glint_survey_api}
\alias{read_glint_survey_api}
\title{Read Viva Glint Survey Data via API}
\usage{
read_glint_survey_api(
  survey_uuid,
  cycle_id,
  emp_id_col = NULL,
  first_name_col = "First Name",
  last_name_col = "Last Name",
  email_col = "Email",
  status_col = "Status",
  completion_date_col = "Survey Cycle Completion Date",
  sent_date_col = "Survey Cycle Sent Date",
  start_date = NULL,
  end_date = NULL,
  encoding = "UTF-8",
  poll_interval = 10,
  max_attempts = 60,
  experience_name = NULL
)
}
\arguments{
\item{survey_uuid}{Survey UUID from Viva Glint}

\item{cycle_id}{Survey cycle ID}

\item{emp_id_col}{Character string specifying the employee ID column name}

\item{first_name_col}{Column name for first name (default: "First Name")}

\item{last_name_col}{Column name for last name (default: "Last Name")}

\item{email_col}{Column name for email (default: "Email")}

\item{status_col}{Column name for status (default: "Status")}

\item{completion_date_col}{Column name for survey completion date
(default: "Survey Cycle Completion Date")}

\item{sent_date_col}{Column name for survey sent date
(default: "Survey Cycle Sent Date")}

\item{start_date}{Optional start date/time for the export window. Can be a
character string in ISO 8601 format, or a Date/POSIXct value.}

\item{end_date}{Optional end date/time for the export window. Can be a
character string in ISO 8601 format, or a Date/POSIXct value.}

\item{encoding}{Character string specifying file encoding (default: "UTF-8")}

\item{poll_interval}{Seconds to wait between status checks (default: 10)}

\item{max_attempts}{Maximum number of polling attempts (default: 60)}

\item{experience_name}{Optional Viva Glint experience name to override the
GLINT_EXPERIENCE_NAME environment variable}
}
\value{
A \code{glint_survey} object (same structure as \code{read_glint_survey}).
The \code{metadata$file_path} field is set to \code{NA}.
}
\description{
Exports a survey cycle through the Microsoft Graph beta API, downloads the
resulting ZIP archive, and returns a \code{glint_survey} object. This is an
alternative to \code{read_glint_survey()} when you want to pull data directly
from Viva Glint instead of importing a local CSV export.
}
\examples{
\dontrun{
glint_setup(
  tenant_id = "your-tenant-id",
  client_id = "your-client-id",
  client_secret = "your-client-secret",
  experience_name = "your-experience-name"
)

survey <- read_glint_survey_api(
  survey_uuid = "your-survey-uuid",
  cycle_id = "your-cycle-id",
  emp_id_col = "EMP ID"
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vivaglint-package.R
\docType{package}
\name{vivaglint-package}
\alias{vivaglint}
\alias{vivaglint-package}
\title{vivaglint: Analysis Tools for 'Viva Glint' Survey Data}
\description{
Provides functions for importing, validating, and analyzing 'Viva Glint' survey data exports, with optional API-based import via the 'Microsoft Graph' API. Includes tools for data reshaping, question-level analysis, multi-cycle comparisons, organizational hierarchy analysis, factor analysis, and correlation analysis. Harman (1960, ISBN: 0226316513); Husser (2017) \doi{10.1002/9781118901731.iecrm0048}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/microsoft/vivaglint}
  \item Report bugs at \url{https://github.com/microsoft/vivaglint/issues}
}

}
\author{
\strong{Maintainer}: Eric Knudsen \email{ericknudsen@microsoft.com}

Other contributors:
\itemize{
  \item Microsoft Corporation [copyright holder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api.R
\name{glint_env}
\alias{glint_env}
\title{Read a required Glint env var, stopping with a helpful message if missing.}
\usage{
glint_env(var_name, label)
}
\description{
Read a required Glint env var, stopping with a helpful message if missing.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hierarchy.R
\name{aggregate_by_manager}
\alias{aggregate_by_manager}
\title{Aggregate Responses by Manager}
\usage{
aggregate_by_manager(
  survey,
  scale_points,
  emp_id_col = NULL,
  manager_id_col,
  full_tree = FALSE,
  plot = FALSE
)
}
\arguments{
\item{survey}{A glint_survey object or data frame containing survey data}

\item{scale_points}{Integer specifying the number of scale points (2-11)}

\item{emp_id_col}{Character string specifying the employee ID column name}

\item{manager_id_col}{Character string specifying the manager ID column name}

\item{full_tree}{Logical indicating whether to include full subtree
(all indirect reports) or only direct reports (default: FALSE)}

\item{plot}{Logical. If \code{TRUE}, prints a ranked lollipop chart of team
Glint Scores by manager and returns the data invisibly. When multiple
questions are present the chart is faceted by question. Requires
\pkg{ggplot2}. Default: \code{FALSE}.}
}
\value{
A tibble with one row per manager-question combination containing:
\describe{
\item{manager_id}{The manager's employee ID}
\item{manager_name}{The manager's full name (First Name + Last Name)}
\item{question}{The question text}
\item{team_size}{Number of employees in the team (direct reports only, or
full subtree when \code{full_tree = TRUE})}
\item{mean, sd, glint_score, n_responses, n_skips, n_total}{Descriptive
statistics for this manager's team on this question}
\item{pct_favorable, pct_neutral, pct_unfavorable}{Favorability
percentages for this manager's team on this question}
}
When \code{plot = TRUE}, the same tibble is returned invisibly after
printing the plot.
}
\description{
Rolls up survey responses to the manager level, calculating the same
metrics as summarize_survey() for each manager's team.
}
\examples{
survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")

# Direct reports only
manager_summary <- aggregate_by_manager(survey, scale_points = 5,
                                        emp_id_col = "EMP ID",
                                        manager_id_col = "Manager ID")

# Full organizational tree
manager_summary_full <- aggregate_by_manager(survey, scale_points = 5,
                                             emp_id_col = "EMP ID",
                                             manager_id_col = "Manager ID",
                                             full_tree = TRUE)

# With ranked dot plot
if (requireNamespace("ggplot2", quietly = TRUE)) {
  aggregate_by_manager(survey, scale_points = 5, emp_id_col = "EMP ID",
                       manager_id_col = "Manager ID", plot = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hierarchy.R
\name{get_all_reports}
\alias{get_all_reports}
\title{Get All Reports for a Manager}
\usage{
get_all_reports(manager_id, data, emp_id_col, manager_id_col)
}
\arguments{
\item{manager_id}{Character string of manager's employee ID}

\item{data}{Data frame with survey data}

\item{emp_id_col}{Character string specifying the employee ID column name}

\item{manager_id_col}{Character string specifying the manager ID column name}
}
\value{
Character vector of employee IDs for all reports
}
\description{
Internal function to get all direct and indirect reports for a manager.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_standard_columns}
\alias{get_standard_columns}
\title{Get Standard Column Names}
\usage{
get_standard_columns(
  emp_id_col,
  first_name_col = "First Name",
  last_name_col = "Last Name",
  email_col = "Email",
  status_col = "Status",
  completion_date_col = "Survey Cycle Completion Date",
  sent_date_col = "Survey Cycle Sent Date"
)
}
\arguments{
\item{emp_id_col}{Character string specifying the employee ID column name}

\item{first_name_col}{Column name for first name (default: "First Name")}

\item{last_name_col}{Column name for last name (default: "Last Name")}

\item{email_col}{Column name for email (default: "Email")}

\item{status_col}{Column name for status (default: "Status")}

\item{completion_date_col}{Column name for survey completion date
(default: "Survey Cycle Completion Date")}

\item{sent_date_col}{Column name for survey sent date
(default: "Survey Cycle Sent Date")}
}
\value{
Character vector of standard column names
}
\description{
Returns the vector of standard column names expected in Viva Glint exports.
The employee ID column name is configurable because customers use different
names. "Manager ID" is included for question-detection exclusion but is not
required to be present.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/api.R
\name{glint_setup}
\alias{glint_setup}
\title{Configure Viva Glint API credentials}
\usage{
glint_setup(
  tenant_id,
  client_id,
  client_secret,
  experience_name,
  save_to_renviron = FALSE
)
}
\arguments{
\item{tenant_id}{Azure AD tenant ID}

\item{client_id}{Azure AD app (client) ID}

\item{client_secret}{Azure AD app client secret}

\item{experience_name}{Viva Glint experience name (e.g., "contoso@demo")}

\item{save_to_renviron}{Logical; if TRUE, append values to ~/.Renviron}
}
\value{
Invisibly returns TRUE after saving credentials
}
\description{
Stores Viva Glint API credentials in environment variables for the current
R session. Optionally writes the values to ~/.Renviron for persistence.
}
\details{
Required environment variables:
\itemize{
\item GLINT_TENANT_ID
\item GLINT_CLIENT_ID
\item GLINT_CLIENT_SECRET
\item GLINT_EXPERIENCE_NAME
}
}
\examples{
\dontrun{
glint_setup(
  tenant_id = "your-tenant-id",
  client_id = "your-client-id",
  client_secret = "your-client-secret",
  experience_name = "your-experience-name",
  save_to_renviron = TRUE
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_favorability_map}
\alias{get_favorability_map}
\title{Get Favorability Map for Scale Points}
\usage{
get_favorability_map(scale_points)
}
\arguments{
\item{scale_points}{Integer specifying the number of scale points (2-11)}
}
\value{
A list with three elements: favorable, neutral, and unfavorable,
each containing a vector of scale points classified into that category
}
\description{
Returns the favorability classification for a given scale point count,
based on Glint standards.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyze.R
\name{get_correlations}
\alias{get_correlations}
\title{Calculate Question Correlations}
\usage{
get_correlations(
  survey,
  method = "pearson",
  format = "long",
  use = "pairwise.complete.obs",
  plot = FALSE
)
}
\arguments{
\item{survey}{A glint_survey object or data frame containing survey data}

\item{method}{Character string indicating the correlation method: "pearson"
(default), "spearman", or "kendall"}

\item{format}{Character string indicating output format: "long" for long format
with one row per question pair (default), or "matrix" for traditional
correlation matrix}

\item{use}{Character string indicating how to handle missing values, passed to
cor() function (default: "pairwise.complete.obs")}

\item{plot}{Logical. If \code{TRUE}, prints a correlation heatmap and returns
the data invisibly. Only supported when \code{format = "long"} (the
default); ignored with a warning when \code{format = "matrix"}. Requires
\pkg{ggplot2}. Default: \code{FALSE}.}
}
\value{
If format = "long", a tibble with columns:
\describe{
\item{question1}{First question text}
\item{question2}{Second question text}
\item{correlation}{Correlation coefficient}
\item{p_value}{P-value for test of correlation significance}
\item{n}{Number of complete pairs used in calculation}
}
If format = "matrix", a matrix with questions as rows and columns.
When \code{plot = TRUE} and \code{format = "long"}, the tibble is returned
invisibly after printing the plot.
}
\description{
Calculates correlations between all question response columns in the survey.
Supports multiple correlation methods and output formats.
}
\examples{
survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")

# Get Pearson correlations in long format (default)
correlations <- get_correlations(survey)

# Get Spearman correlations
correlations_spearman <- get_correlations(survey, method = "spearman")

# Get correlation matrix
cor_matrix <- get_correlations(survey, format = "matrix")

# With correlation heatmap
if (requireNamespace("ggplot2", quietly = TRUE)) {
  get_correlations(survey, plot = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reshape.R
\name{pivot_long}
\alias{pivot_long}
\title{Pivot Survey Data to Long Format}
\usage{
pivot_long(
  survey,
  data_type = "all",
  include_empty = FALSE,
  include_standard_cols = TRUE
)
}
\arguments{
\item{survey}{A glint_survey object or data frame containing survey data}

\item{data_type}{Character string indicating what data to return: "all" for all
responses including those without comments, "comments" for only responses with
comments, or "both" for separate tibbles (default: "all")}

\item{include_empty}{Logical indicating whether to include empty comments when
data_type = "comments" (default: FALSE)}

\item{include_standard_cols}{Logical indicating whether to include standard
columns (EMP ID, Manager ID, etc.) in the output (default: TRUE)}
}
\value{
When \code{data_type = "all"} or \code{"comments"}, a single tibble
in long format with columns:
\describe{
\item{Standard columns}{EMP ID, First Name, Last Name, etc. (if
\code{include_standard_cols = TRUE})}
\item{question}{The question text}
\item{response}{Numeric response value}
\item{comment}{Comment text (NA if not provided)}
\item{comment_topics}{Comma-separated topic tags (NA if not provided)}
\item{sensitive_flag}{Sensitivity flag value}
}
When \code{data_type = "both"}, a named list with two elements:
\describe{
\item{all}{Long-format tibble of all responses (same structure as above)}
\item{comments}{Long-format tibble filtered to rows with non-empty comments}
}
}
\description{
Transforms survey data from wide format (one row per respondent) to long format
(one row per respondent-question combination). Can return all responses, only
comments, or both depending on the data_type parameter.
}
\examples{
survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")

# Get all responses (with and without comments)
all_data <- pivot_long(survey)

# Get only responses with comments
comments_only <- pivot_long(survey, data_type = "comments")

# Get both separately
both <- pivot_long(survey, data_type = "both")
all_responses <- both$all
comments <- both$comments
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyze.R
\name{analyze_by_attributes}
\alias{analyze_by_attributes}
\title{Analyze Survey Responses by Attributes}
\usage{
analyze_by_attributes(
  survey,
  attribute_file = NULL,
  scale_points,
  attribute_cols,
  emp_id_col = NULL,
  min_group_size = 5,
  plot = FALSE
)
}
\arguments{
\item{survey}{A \code{glint_survey} object or data frame. If attributes have
already been joined via \code{join_attributes()}, \code{attribute_file} can
be omitted.}

\item{attribute_file}{Optional. A file path (character string) or data frame
containing employee attributes to join. If \code{NULL} (default), the
survey must already contain the columns named in \code{attribute_cols}.}

\item{scale_points}{Integer specifying the number of scale points (2-11)}

\item{attribute_cols}{Character vector of column names to group by
(e.g., \code{c("Department", "Gender", "Tenure Group")})}

\item{emp_id_col}{Character string specifying the employee ID column name
in both the survey data and the attribute data}

\item{min_group_size}{Integer specifying the minimum number of employees
required for a group to be included in results (default: 5)}

\item{plot}{Logical. If \code{TRUE}, prints a faceted dot plot of Glint
Scores by attribute group (one facet per question) and returns the data
invisibly. When multiple \code{attribute_cols} are supplied, only the first
is plotted. Requires \pkg{ggplot2}. Default: \code{FALSE}.}
}
\value{
A tibble with one row per attribute-group-question combination
containing:
\describe{
\item{attribute columns}{Values for each attribute grouping variable}
\item{group_size}{Number of employees in this attribute group}
\item{question, mean, sd, n_responses, n_skips, n_total}{
Descriptive statistics for this group on this question}
\item{pct_favorable, pct_neutral, pct_unfavorable}{Favorability
percentages for this group on this question}
}
When \code{plot = TRUE}, the same tibble is returned invisibly after
printing the plot.
}
\description{
Aggregates survey responses by employee attributes (e.g., Department,
Gender, Tenure) and calculates the same metrics as \code{summarize_survey()}
for each attribute group combination. Only groups meeting the minimum size
threshold are included in the results.
}
\details{
Attributes can be supplied in two ways:
\enumerate{
\item Pass \code{attribute_file} (a file path or data frame) and the join
is performed internally on each call.
\item Pre-join attributes once with \code{join_attributes()} and pass the
enriched survey directly — omit \code{attribute_file} entirely. This is
more efficient when calling \code{analyze_by_attributes()} multiple times
or when you want to filter the data before analysis.
}
}
\examples{
survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
attr_file <- system.file("extdata", "employee_attributes.csv", package = "vivaglint")
survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")

# Option 1: provide attribute_file directly
results <- analyze_by_attributes(
  survey,
  attribute_file = attr_file,
  scale_points = 5,
  attribute_cols = "Department",
  emp_id_col = "EMP ID",
  min_group_size = 2
)

# Option 2: pre-join with join_attributes(), then omit attribute_file
survey_enriched <- join_attributes(survey, attr_file, emp_id_col = "EMP ID")
results <- analyze_by_attributes(survey_enriched, scale_points = 5,
                                 attribute_cols = "Department",
                                 emp_id_col = "EMP ID",
                                 min_group_size = 2)

# With dot plot
if (requireNamespace("ggplot2", quietly = TRUE)) {
  analyze_by_attributes(survey_enriched, scale_points = 5,
                        attribute_cols = "Department",
                        emp_id_col = "EMP ID", plot = TRUE,
                        min_group_size = 2)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{mean_to_glint_score}
\alias{mean_to_glint_score}
\title{Convert Raw Mean to Glint Score}
\usage{
mean_to_glint_score(mean_val, scale_points)
}
\arguments{
\item{mean_val}{Numeric raw mean value}

\item{scale_points}{Integer number of scale points (2-11)}
}
\value{
Numeric Glint Score on a 0-100 scale
}
\description{
Transforms a raw mean response on a rating scale to the 100-point Glint Score
as displayed in the Viva Glint UI. Rounded to the nearest whole number, consistent
with Glint's rounding convention.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyze.R
\name{split_survey_data}
\alias{split_survey_data}
\title{Split Survey Data into Quantitative and Qualitative Components}
\usage{
split_survey_data(survey, emp_id_col = NULL)
}
\arguments{
\item{survey}{A glint_survey object or data frame containing survey data}

\item{emp_id_col}{Character string specifying the employee ID column name}
}
\value{
A named list with two elements:
\describe{
\item{quantitative}{A tibble with all standard respondent columns plus
one numeric response column per question. Comment, topic, and sensitive
flag columns are excluded.}
\item{qualitative}{A tibble with the employee ID column plus the comment,
topic, and sensitive flag columns for every question. Numeric response
columns are excluded.}
}
}
\description{
Separates a survey dataset into two data frames: one containing only numeric
response data (for statistical analysis) and one containing only comment
and topic data (for qualitative analysis). Both outputs retain the employee
ID column so they can be rejoined at any time.
}
\examples{
survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")
parts  <- split_survey_data(survey)

# Analyze numeric responses
summary <- summarize_survey(parts$quantitative, scale_points = 5,
                            emp_id_col = "EMP ID")

# Work with comments separately
comments <- parts$qualitative
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{parse_comment_topics}
\alias{parse_comment_topics}
\title{Parse Comment Topics}
\usage{
parse_comment_topics(topics, return_format = "list")
}
\arguments{
\item{topics}{Character vector of comma-separated topic strings}

\item{return_format}{Character string specifying output format: "list" (default) or "tidy"}
}
\value{
If return_format is "list", returns a list of character vectors.
If return_format is "tidy", returns a tibble with columns for original index and individual topics.
}
\description{
Splits comma-separated topic strings into vectors, handling empty and NA values.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{join_attributes}
\alias{join_attributes}
\title{Join Employee Attributes to Survey Data}
\usage{
join_attributes(survey, attribute_source, emp_id_col = NULL)
}
\arguments{
\item{survey}{A \code{glint_survey} object or data frame containing survey data}

\item{attribute_source}{Either a character string file path to a CSV file, or
a data frame containing employee attributes. All columns are coerced to
character to avoid type conflicts during joining.}

\item{emp_id_col}{Character string specifying the employee ID column name.
Must match the column name in both the survey data and the attribute data.
When \code{survey} is a \code{glint_survey} object this defaults to
\code{survey$metadata$emp_id_col} (set at import via
\code{read_glint_survey()}), so you can omit this argument if you loaded
the survey with the correct \code{emp_id_col}.}
}
\value{
If \code{survey} is a \code{glint_survey} object, returns an enriched
\code{glint_survey} with the attribute columns appended to \code{$data} and
the names of all joined attribute columns stored in
\code{$metadata$attribute_cols}. If \code{survey} is a plain data frame,
returns the joined data frame.

Respondents with no match in the attribute data will have \code{NA} for all
attribute columns; a message is emitted indicating how many were affected.
}
\description{
Reads employee attribute data from a CSV file or data frame and joins it to
a survey object by employee ID. Returns an enriched \code{glint_survey}
object that can be passed directly to \code{analyze_by_attributes()} or any
other package function, eliminating the need to re-read and re-join the
attribute file on every call.
}
\examples{
survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
attr_path <- system.file("extdata", "employee_attributes.csv", package = "vivaglint")
survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")

survey_enriched <- join_attributes(survey, attr_path, emp_id_col = "EMP ID")
results <- analyze_by_attributes(survey_enriched, scale_points = 5,
                                 attribute_cols = "Department")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyze.R
\name{extract_survey_factors}
\alias{extract_survey_factors}
\title{Extract Survey Factors}
\usage{
extract_survey_factors(
  survey,
  n_factors = NULL,
  rotation = "oblimin",
  min_loading = 0.3,
  fm = "minres",
  plot = FALSE
)
}
\arguments{
\item{survey}{A glint_survey object or data frame containing survey data}

\item{n_factors}{Integer indicating the number of factors to extract. If NULL
(default), will use parallel analysis to determine optimal number of factors}

\item{rotation}{Character string indicating rotation method: "oblimin" (default),
"varimax", "promax", "quartimax", "equamax", or "none"}

\item{min_loading}{Minimum factor loading to display in results (default: 0.3)}

\item{fm}{Character string indicating factoring method: "minres" (minimum
residuals, default), "ml" (maximum likelihood), "pa" (principal axis),
"wls" (weighted least squares), "gls" (generalized least squares), or
"uls" (unweighted least squares)}

\item{plot}{Logical. If \code{TRUE}, prints a factor loading heatmap and
returns the result list invisibly. Requires \pkg{ggplot2}. Default:
\code{FALSE}.}
}
\value{
A list containing:
\describe{
\item{factor_summary}{A tibble with one row per question-factor combination
(filtered to \code{abs(loading) >= min_loading}), containing:
\code{question} (item text), \code{factor} (factor name),
\code{loading} (loading coefficient), \code{loading_label}
("Strong" >= 0.75 / "Medium" 0.60-0.74 / "Weak" < 0.60),
\code{communality} (proportion of item variance explained by all factors),
and \code{factor_variance_pct} (percentage of total variance explained
by this factor). Sorted by factor then descending loading strength.}
\item{fa_object}{Original fa object from psych package for further analysis}
}
When \code{plot = TRUE}, the same list is returned invisibly after printing
the plot.
}
\description{
Performs factor analysis on survey question responses to identify underlying
latent factors. Supports multiple rotation methods with oblique rotation as default.
}
\examples{
survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")

if (requireNamespace("psych", quietly = TRUE)) {
  # Extract factors with fixed count to keep runtime small
  factors <- extract_survey_factors(survey, n_factors = 1)
  print(factors$factor_summary)

  # Filter to strong loaders only
  strong <- dplyr::filter(factors$factor_summary, loading_label == "Strong")

  # With factor loading heatmap
  if (requireNamespace("ggplot2", quietly = TRUE)) {
    extract_survey_factors(survey, n_factors = 1, plot = TRUE)
  }
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyze.R
\name{summarize_survey}
\alias{summarize_survey}
\title{Summarize Survey Questions}
\usage{
summarize_survey(
  survey,
  scale_points,
  questions = "all",
  emp_id_col = NULL,
  plot = FALSE
)
}
\arguments{
\item{survey}{A glint_survey object or data frame containing survey data}

\item{scale_points}{Integer specifying the number of scale points (2-11)}

\item{questions}{Character vector of question text(s) to analyze, or "all" to
analyze all questions (default: "all")}

\item{emp_id_col}{Character string specifying the employee ID column name}

\item{plot}{Logical. If \code{TRUE}, prints a favorability stacked bar chart
sorted by Glint Score and returns the data invisibly. Requires
\pkg{ggplot2}. Default: \code{FALSE}.}
}
\value{
A tibble with one row per question containing:
\describe{
\item{question}{The question text}
\item{mean}{Mean of numeric responses (raw scale)}
\item{sd}{Standard deviation of numeric responses}
\item{glint_score}{Mean transformed to a 0-100 scale, matching the score
displayed in the Viva Glint UI: \code{round(((mean - 1) / (scale_points - 1)) * 100)}}
\item{n_responses}{Count of non-blank, non-null responses}
\item{n_skips}{Count of blank or null responses}
\item{n_total}{Total number of respondents}
\item{pct_favorable}{Percentage of responses classified as favorable}
\item{pct_neutral}{Percentage of responses classified as neutral}
\item{pct_unfavorable}{Percentage of responses classified as unfavorable}
}
When \code{plot = TRUE}, the same tibble is returned invisibly after
printing the plot.
}
\description{
Calculates comprehensive metrics for survey questions, returning a summary
analysis in tidy format. This includes mean, standard deviation, Glint Score,
response counts, skip counts, and favorability percentages.
}
\examples{
survey_path <- system.file("extdata", "survey_export.csv", package = "vivaglint")
survey <- read_glint_survey(survey_path, emp_id_col = "EMP ID")

# Summarize all questions (5-point scale)
summary <- summarize_survey(survey, scale_points = 5)

# Summarize specific questions
summary_subset <- summarize_survey(survey, scale_points = 5,
  questions = c("My work is meaningful", "I feel valued"))

# With favorability chart
if (requireNamespace("ggplot2", quietly = TRUE)) {
  summarize_survey(survey, scale_points = 5, plot = TRUE)
}
}
